//
//  TITLE  : scene_lighting.cpp 
//
//  AUTHOR : Robert Bateman
//
//  DATE   : 10 Feb 2001
//
//  NOTES  : Contains a series of classes that help to define a given
//           light. It allows you to run seperate specualr passes, as well
//           as your own lighting equations. This forms the basis to quick
//           lighting calculations of background scenery( using pre-calculation )
//           as well as using simpler( &quicker ) realtime lighting calculations
//
//  LAST_UPDATED : 28 March 2001
//

//______________________________________________________________ Includes

#include "./scene_lighting.h"

//______________________________________________________________ Clamp Value Of Colour

void clamp( float *col )
  {
    if( col[0] > 1 ) col[0] = 1;
    if( col[1] > 1 ) col[1] = 1;
    if( col[2] > 1 ) col[2] = 1;
    if( col[0] < 0 ) col[0] = 0;
    if( col[1] < 0 ) col[1] = 0;
    if( col[2] < 0 ) col[2] = 0;
  }

//______________________________________________________________ Set Light Colour Funcs

void cAmbientLight::setLightAmbient( float *col )
  {
    if( col )
      {
        pAmbient[0] = col[0];
        pAmbient[1] = col[1];
        pAmbient[2] = col[2];
        clamp( pAmbient );
      }
  }

void cDirectionalLight::setLightDiffuse( float *col )
  {
    if( col )
      {
        pDiffuse[0] = col[0];
        pDiffuse[1] = col[1];
        pDiffuse[2] = col[2];
        clamp( pDiffuse );
      }
  }

void cPointLight::setLightSpecular( float *col )
  {
    if( col )
      {
        pSpecular[0] = col[0];
        pSpecular[1] = col[1];
        pSpecular[2] = col[2];
        clamp( pSpecular );
      }
  }

//______________________________________________________________ Get Light Colour Funcs

float *cAmbientLight::getLightAmbient( void )
  {
    return &pAmbient[0];
  }

float *cDirectionalLight::getLightDiffuse( void )
  {
    return &pDiffuse[0];
  }

float *cPointLight::getLightSpecular( void )
  {
    return &pSpecular[0];
  }

//______________________________________________________________ Set Pixel Fragment Colour Funcs

float cAmbientLight::setFragmentColour( cVector3f &pos ,
                                        cVector3f &nml ,
                                        float     *col )
  {
    float d = ((*this) - pos).length();
    float Afactor = getAttenuationFactor( d );
    col[0] = pAmbient[0] * Afactor ;
    col[1] = pAmbient[1] * Afactor ;
    col[2] = pAmbient[2] * Afactor ;
    return d;
  }


float cDirectionalLight::setFragmentColour( cVector3f &pos ,
                                            cVector3f &nml ,
                                            float     *col )
  {
    float diffuse_term = pDirection.normalized() * nml.normalized() ;
    col[0] = pAmbient[0] + diffuse_term*pDiffuse[0] ;
    col[1] = pAmbient[1] + diffuse_term*pDiffuse[1] ;
    col[2] = pAmbient[2] + diffuse_term*pDiffuse[2] ;
    clamp( col );
    return pos.length();
  }



float cPointLight::setFragmentColour( cVector3f &pos ,
                                      cVector3f &nml ,
                                      float     *col )
  {
    cVector3f lpos( x , y , z ) ,lv;
    lv =  (*this) - pos;
    float d = lv.length(),
    att  = getAttenuationFactor( d ); ;

    // diffuse term is dot product between light vector & normal @ vertex
    float diffuse_term = lv.normalized()*nml.normalized() ;
    col[0] = (pAmbient[0] + diffuse_term*pDiffuse[0])*att ;
    col[1] = (pAmbient[1] + diffuse_term*pDiffuse[1])*att ;
    col[2] = (pAmbient[2] + diffuse_term*pDiffuse[2])*att ;
    clamp( col );
    return d;
  }


//______________________________________________________________ Set Pixel Fragment Specular Colour

float cPointLight::setSpecularColourFragment( cVector3f &pos ,
                                              cVector3f &nml ,
                                              float     *col ,
                                              float      shi )
  //
  //  Specular Lighting calculation :: Is = Ii(R.V)^n
  //

  {
     cVector3f V = ( pos - camPos  ).normalized();
     cVector3f L = ( pos - (*this) );
     cVector3f N = nml.normalized();

     float att , l;
     l = L.length();
     att  = getAttenuationFactor( l );
     L.normalize();

     // R = (-L) + 2Ncos

     cVector3f R = N*2*cos(N*L) - L ;

           
     float R_dot_V_to_power_n = pow( (R*V) , shi );

     col[0] = att * pSpecular [0] * R_dot_V_to_power_n;
     col[1] = att * pSpecular [1] * R_dot_V_to_power_n;
     col[2] = att * pSpecular [2] * R_dot_V_to_power_n;
     return 0;
   }


