//
//  TITLE  : scene_lighting.h 
//
//  AUTHOR : Robert Bateman
//
//  DATE   : 10 Feb 2001
//
//  NOTES  : Contains a series of classes that help to define a given
//           light. It allows you to run seperate specualr passes, as well
//           as your own lighting equations. This forms the basis to quick
//           lighting calculations of background scenery( using pre-calculation )
//           as well as using simpler( &quicker ) realtime lighting calculations
//
//  LAST_UPDATED : 28 March 2001
//


//______________________________________________________________ Includes

#ifndef LIGHTING_H
#define LIGHTING_H

#include "./fruity_math.h"
#include "./fruity_exception.h"
#include "./opengl_includes.h"

//______________________________________________________________ Defs

#define CONSTANT  0
#define LINEAR    1
#define QUADRATIC 2

#ifndef PI
#define PI 3.1459
#endif


//______________________________________________________________ Camera Vector Position

  // camera position variable held in mayamouse.h
extern cVector3f camPos;

extern void clamp( float *col );

//______________________________________________________________ cLight Base Class

class cLight : public cVector3f

  //
  //  base class definition for fake & pre-calculated light sources
  //  only defines attenuation. Derived from cVector3f to simplify 
  //  movement & positioning of light & also shadowing

  {

    public:

      cLight( void ): cVector3f( )
        {
          fConstantAttenuation  =    1 ;
          fLinearAttenuation    =    0 ;
          fQuadraticAttenuation =    0 ;
          bEnabled              = true ;
        };

      void enable( void ) {
          bEnabled = true;
        }
      void disable( void ){
          bEnabled = false;
        }
      float getAttenuationFactor( float &d ){
            return divide_f( 1 , (fConstantAttenuation)   +
                                 (d*fLinearAttenuation)   +
                                 (d*d*fQuadraticAttenuation) );
        }

      virtual float setFragmentColour( cVector3f &pos ,
                                       cVector3f &nml ,
                                       float     *col )
      { return 0; };

      float  fConstantAttenuation  ;
      float  fLinearAttenuation    ;
      float  fQuadraticAttenuation ;

      bool   bEnabled;

  };

//______________________________________________________________ Simple Ambient Light

class cAmbientLight : public cLight

  //
  // This class adds basic ambient lighting calculations with attenuation 
  //

  {

    public :

      cAmbientLight( void ) : cLight()
        {
          pAmbient[0] = 0.0;
          pAmbient[1] = 0.0;
          pAmbient[2] = 0.0;
        };

      void           setLightAmbient  ( float * );
      float         *getLightAmbient  ( void    );
      virtual float  setFragmentColour( cVector3f &pos ,
                                        cVector3f &nml ,
                                        float     *col );
      float pAmbient[4];

  };

//______________________________________________________________ Simple Directional (diffuse) Light

class cDirectionalLight : public cAmbientLight

  //
  //  This class adds basic diffuse lighting calculations
  //  with a given direction vector....
  //

  {

    public :

      cDirectionalLight( void ):cAmbientLight()
        {
          pDiffuse [0] = pDiffuse [1] = pDiffuse [2] = 0.6;
        };

      void   setLightDiffuse ( float * );
      float *getLightDiffuse ( void    );

      inline void setLightDirection( cVector3f &direction ) {
          pDirection = direction;
        }

      inline cVector3f getLightDirection( void ) {
          return pDirection;
        }

      virtual float setFragmentColour( cVector3f &pos , cVector3f &nml , float *col );


      float      pDiffuse  [4];
      cVector3f  pDirection;

  };

//______________________________________________________________ Point Light

class cPointLight : public cDirectionalLight

  //
  //  This class adds specular lighting calculations
  //  and point light behaviour.
  //
  //  NOTE: specular lighting is a seperate calculation
  //

  {

    public :

      cPointLight( void ) : cDirectionalLight()
        {
          pSpecular[0] = pSpecular[1] = pSpecular[2] = 1.0;
        };



      void   setLightSpecular( float * );
      float *getLightSpecular( void    );

      virtual float setFragmentColour        ( cVector3f &pos , 
                                               cVector3f &nml ,
                                               float     *col );
      virtual float setSpecularColourFragment( cVector3f &pos ,
                                               cVector3f &nml ,
                                               float     *col ,
                                               float      shi = 15);
      float  pSpecular [4];
  };


#endif
