//
//
//  TITLE : objloader.cpp ( for boona taxi )
//
//  AUTHORS : Robert Bateman & Mike Bonnington
//
//  NOTES :
//
//          Trying to get nice fast poly display + multiple render passes for gloss
//          & environment mapping + adding in extra maths functionality for lighting
//          calculations + bsp generation.
//
//
//

//______________________________________________________________________________ Includes

#include <string.h>
#include <iostream.h>
#include <fstream.h>

#include "./objloader.h"

//______________________________________________________________________________ Global Data

//
//  Global Light Vector, used in 'faked' lighting calculations
//

cVector3f global_light( -1.5 , -2 , 0.5 );

//
//  Global switches to turn rendering effects on or off
//

bool   useEnv      = true ,
       useGloss    = true ,
       useLighting = true ;

GLuint _setLight;


//______________________________________________________________________________ init Renderer Function


bool initRenderer( void )

  //
  //  sets up display lists and depth buffering for drawing multiple passes
  //

  {
    static float pAmbient [] = { 0.3 , 0.3  , 0.3 , 1.0 },
                 pDiffuse [] = { 0.7 , 0.7  , 0.7 , 1.0 },
                 pSpecular[] = { 1.0 , 1.0  , 1.0 , 1.0 },
                 pPosition[] = { 0.0 , 10.0 , 1.0 , 1.0 };
    _setLight = glGenLists( 1 );

    glNewList( _setLight , GL_COMPILE );

      glLightfv( GL_LIGHT0 , GL_AMBIENT  , pAmbient  );
      glLightfv( GL_LIGHT0 , GL_AMBIENT  , pDiffuse  );
      glLightfv( GL_LIGHT0 , GL_AMBIENT  , pSpecular );
      glLightfv( GL_LIGHT0 , GL_POSITION , pPosition );

      glEnable( GL_LIGHTING );
      glEnable( GL_LIGHT0   );

      glMaterialfv( GL_FRONT , GL_AMBIENT   , pAmbient  );
      glMaterialfv( GL_FRONT , GL_DIFFUSE   , pDiffuse  );
      glMaterialfv( GL_FRONT , GL_SPECULAR  , pSpecular );
      glMaterialf ( GL_FRONT , GL_SHININESS , 5 ); 
    glEndList();

    glDepthFunc( GL_LEQUAL );
    return true;

  }

//______________________________________________________________________________ Triangle Constructor

Triangle::Triangle( void ): plane4f()

  //
  // Sets all data pointers to NULL
  //

  {
    t1 = NULL;
    t2 = NULL;
    t3 = NULL;
    n1 = NULL;
    n2 = NULL;
    n3 = NULL;
    v1 = NULL;
    v2 = NULL;
    v3 = NULL;
    sibling = NULL;
    child   = NULL;
  };

//______________________________________________________________________________ Plane Calculation Function

void Triangle::setup( void )

  //
  //  This function calculates a polygons plane eqation
  //

  {
    plane4f temp( *v1 , *v2 , *v3 );
    a = temp.a;
    b = temp.b;
    c = temp.c;
    d = temp.d;
  }





//______________________________________________________________________________ Lighting Calculation Function

void Triangle::calc_light( void )

  //
  //     This function calculates the colour at each vertex 
  //  by performing a dot product with the normal & light vector
  //

  {
    float dot_product;
    dot_product = global_light * (*n1);
    angle1 = divide_f( dot_product , ((*n1).length() * global_light.length()) );
    angle1 = 0.5 + divide_f( acos( angle1 ) , 2*(PI) );

    dot_product = global_light * (*n2);
    angle2 = divide_f( dot_product , ((*n2).length() * global_light.length()) );
    angle2 = 0.5 + divide_f( acos( angle2 ) , 2*(PI) );

    dot_product = global_light * (*n3);
    angle3 = divide_f( dot_product , ((*n3).length() * global_light.length()) );
    angle3 = 0.5 + divide_f( acos( angle3 ) , 2*(PI) );
  }




//______________________________________________________________________________ objFile Constructor

objFile::objFile( void )

  //
  //  Initialise counts & set Pointers to NULL
  //

  {
    vertexCount   = 0;
    normalCount   = 0;
    texCoordCount = 0;
    triangleCount = 0;
    MaterialCount = 0;
    vertexArray   = NULL;
    normalArray   = NULL;
    texCoordArray = NULL;
    triangleArray = NULL;
    MaterialArray = NULL;
  }




//______________________________________________________________________________ Load Obj File Function

bool objFile::load( char *filename )

  //
  //  Loads Vertices, Normals , Uv Coords and Triangles from an obj file
  //  Places them in arrays in the class
  //

  {
    ifstream inputFile( filename );
    uint i;

    if( !inputFile )
      {
        cout << "Error : Unable to open File" << filename 
             << endl;
        return false;
      }


    else
      {

        char In_String[128];

          //
          // count data
          //
        while( !inputFile.eof() )
          {
            inputFile >> In_String;
            
            if( strcmp( In_String , "v"  ) == 0 ) vertexCount++;
            if( strcmp( In_String , "vn" ) == 0 ) normalCount++;
            if( strcmp( In_String , "vt" ) == 0 ) texCoordCount++;
            if( strcmp( In_String , "f"  ) == 0 ) triangleCount++;
            if( strcmp( In_String , "usemtl" ) == 0 ) MaterialCount++;

          }


          // allocate memory and set vertexCount
        vertexArray   = new Vertex  [ vertexCount   ];
        normalArray   = new Normal  [ normalCount   ];
        texCoordArray = new TexCoord[ texCoordCount ];
        triangleArray = new Triangle[ triangleCount ];
        MaterialArray = new material[ MaterialCount ];

          // check to see if memory allocation ok...
        CheckMemory( vertexArray );
        CheckMemory( normalArray );
        CheckMemory( texCoordArray );
        CheckMemory( triangleArray );
        CheckMemory( MaterialArray );
        


          // close & re-open file stream
        inputFile.close();
        inputFile.open( filename );


          // create some temorary count values
        int vCount = 0 ,
            nCount = 0 ,
            tCount = 0 ,
            fCount = 0 ;
        MaterialCount = 0 ;

          // get data and place in array of verticies
        while( !inputFile.eof() )
          {
             inputFile >> In_String;

             if( strcmp( In_String , "v" ) == 0 )
               {
                 inputFile >> vertexArray[ vCount ].x
                           >> vertexArray[ vCount ].y
                           >> vertexArray[ vCount ].z;
                 vCount++;
               }

             if( strcmp( In_String , "vn" ) == 0 )
               {
                 inputFile >> normalArray[ nCount ].x
                           >> normalArray[ nCount ].y
                           >> normalArray[ nCount ].z;
                 nCount++;
               }

             if( strcmp( In_String , "vt" ) == 0 )
               {
                 inputFile >> texCoordArray[ tCount ].u
                           >> texCoordArray[ tCount ].v;
                 tCount++;
               }

             if( strcmp( In_String , "usemtl" ) == 0 )
               {
                 inputFile >> In_String;
                 MaterialCount +=1;
               }

             if( strcmp( In_String , "f" ) == 0 )
               {

                   // get the rest of the line

                 inputFile.getline( In_String , 127, '\n' );


                   // three arrays to hold indicies

                 int Verticies[3];
                 int Normals[3];
                 int TexCoords[2];


                   // remove all slashes from string

                 for( int i=0; i<128; i++ ) 
                      if(In_String[i] == '/') In_String[i] = ' ';



                   // Scan in indicies for verticies, normals & tecoords

                 sscanf( In_String     , "%d%d%d%d%d%d%d%d%d",
                         &Verticies[0] , &TexCoords[0] , &Normals[0] ,
                         &Verticies[1] , &TexCoords[1] , &Normals[1] ,
                         &Verticies[2] , &TexCoords[2] , &Normals[2] );



                   // Need to take 1 off each index number. The *.obj index
                   // values start at 1, but array positions start at zero.

                 for( i=0; i<3; i++ )
                   {
                      triangleArray[ fCount ].vinds[i] = --Verticies[i];
                      --Normals[i];
                      --TexCoords[i];
                   }
                         


                   // get pointers to required element of array

                 triangleArray[ fCount ].v1 = &vertexArray  [ Verticies[0] ];
                 triangleArray[ fCount ].n1 = &normalArray  [ Normals  [0] ];
                 triangleArray[ fCount ].t1 = &texCoordArray[ TexCoords[0] ];

                 triangleArray[ fCount ].v2 = &vertexArray  [ Verticies[1] ];
                 triangleArray[ fCount ].n2 = &normalArray  [ Normals  [1] ];
                 triangleArray[ fCount ].t2 = &texCoordArray[ TexCoords[1] ];

                 triangleArray[ fCount ].v3 = &vertexArray  [ Verticies[2] ];
                 triangleArray[ fCount ].n3 = &normalArray  [ Normals  [2] ];
                 triangleArray[ fCount ].t3 = &texCoordArray[ TexCoords[2] ];

                 triangleArray[ fCount ].setup();
                 triangleArray[ fCount ].calc_light();
                 triangleArray[ fCount ].mat = MaterialCount - 1 ;
                 fCount++;
               }
             
          }

        inputFile.close();
      }

     shadowPositions = new Vertex[ vertexCount ];
     CheckMemory( shadowPositions );

     matrix xForm;
     plane4f GroundPlane( 0 , 1 , 0 , 0 );
     calc_shadows  ( xForm , GroundPlane );


    return true;

  }

void objFile::calc_shadows  ( matrix m , plane4f groundPlane )
  {

    cVector3f endPos , faceNormal = groundPlane.normal();;
    float multiplier;
         
    for( uint i=0; i<vertexCount; i++ )
      {
        endPos = m*vertexArray[ i ];
        multiplier = divide_f( ((endPos * faceNormal) + groundPlane.d) , (global_light * faceNormal) );
        shadowPositions[i] = (endPos - (global_light*multiplier));
      }
  }

void objFile::draw_shadows  ( void )
  {
    glPushMatrix();
      glDisable( GL_DEPTH_TEST );
      glDisable(GL_TEXTURE_2D);
      glColor3f(0.15, 0.15, 0.15);
      glBegin( GL_TRIANGLES );
        for( uint i=0; i<triangleCount; i++ )
          {
            glVertex3f( shadowPositions [ triangleArray[ i ].vinds[0] ].x ,
                        shadowPositions [ triangleArray[ i ].vinds[0] ].y ,
                        shadowPositions [ triangleArray[ i ].vinds[0] ].z );
            glVertex3f( shadowPositions [ triangleArray[ i ].vinds[1] ].x ,
                        shadowPositions [ triangleArray[ i ].vinds[1] ].y ,
                        shadowPositions [ triangleArray[ i ].vinds[1] ].z );
            glVertex3f( shadowPositions [ triangleArray[ i ].vinds[2] ].x ,
                        shadowPositions [ triangleArray[ i ].vinds[2] ].y ,
                        shadowPositions [ triangleArray[ i ].vinds[2] ].z );
          }
      glEnd();
      glEnable( GL_DEPTH_TEST );
      glEnable(GL_TEXTURE_2D);
    glPopMatrix();
  }

//______________________________________________________________________________ Draw Model Function

bool objFile::draw( void )

  {

    //
    // Variables to check for changes in material numbers
    //

    int    curMat = -1;;
    bool   Gloss;


    //  Set up Sphere Mapping

    glTexGeni( GL_S                ,
               GL_TEXTURE_GEN_MODE ,
               GL_SPHERE_MAP       );
    glTexGeni( GL_T                ,
               GL_TEXTURE_GEN_MODE ,
               GL_SPHERE_MAP       );

    glPushMatrix();


      //
      // If The 'useLighting' flag is true, turn on light 
      // and set up material properties
      //

      if( useLighting )
        {
          glCallList( _setLight );
        }


        //
        // First Rendering Pass draws the environment map
        //

        glDisable( GL_BLEND );

        glPushMatrix();
          glBegin(GL_TRIANGLES);

            //
            //  Loop Through all triangles
            //

            for( uint i=0; i<triangleCount; i++ )
              {

                  //
                  // If we need to set up a material 
                  // ( either it has changed or we
                  //   have just started drawing )
                  //

                if( triangleArray[ i ].mat != curMat )
                  {
                    Gloss = false;
                    if( MaterialArray[ triangleArray[ i ].mat ].EnvMap && useEnv  )
                      {
                          glEnd();
                        glPopMatrix();
                        glPushMatrix();
                          Gloss = true;
                          glEnable( GL_TEXTURE_GEN_S );
                          glEnable( GL_TEXTURE_GEN_T );
                          glBindTexture( GL_TEXTURE_2D , MaterialArray[ triangleArray[ i ].mat ].TexObjB );
                          glBegin(GL_TRIANGLES);
                        }
                      curMat = triangleArray[ i ].mat;
                    }

            //
            // only draw the triangles that have environment mapping
            //

            if( Gloss )
              {
                triangleArray[ i ].draw();
              }
          }
        glEnd();
        glPopMatrix();


        //
        // Second Rendering Pass draws the surface map
        //

        curMat = -1;

        glPushMatrix();
          glBegin(GL_TRIANGLES);

            //
            //  Loop Through all triangles
            //

            for( uint i=0; i<triangleCount; i++ )
              {

                //
                // If we need to set up a material 
                // ( either it has changed or we
                //   have just started drawing )
                //

                if( triangleArray[ i ].mat != curMat )
                  {
                    glEnd();
                    glDisable( GL_BLEND );
                  
                    if( MaterialArray[ triangleArray[ i ].mat ].Gloss && useGloss && useEnv )
                      {
                        glBlendFunc( GL_ONE_MINUS_SRC_ALPHA , GL_SRC_ALPHA );
                        glEnable( GL_BLEND );
                      }
                    else
                    if( MaterialArray[ triangleArray[ i ].mat ].EnvMap && useEnv )
                      {
                        glBlendFunc( GL_ONE , GL_ONE );
                        glEnable( GL_BLEND );
                      }

                    glDisable( GL_TEXTURE_GEN_S    );
                    glDisable( GL_TEXTURE_GEN_T    );
                    glBindTexture( GL_TEXTURE_2D , MaterialArray[ triangleArray[ i ].mat ].TexObjA );

                    glBegin(GL_TRIANGLES);
                    curMat = triangleArray[ i ].mat;
                  }
                triangleArray[ i ].draw();
              }
          glEnd();
        glPopMatrix();

        if( useLighting )
          {
            glDisable( GL_LIGHTING );
            glDisable( GL_LIGHT0 );
          }

      glPopMatrix();

    glDisable( GL_BLEND );

    return true;

  }




//______________________________________________________________________________ objFile Destructor

objFile::~objFile(void)

  //
  // Delete all dynamically allocated data
  //

  {
     if( vertexArray != NULL )
       {
          delete [] vertexArray;
          delete [] normalArray;
          delete [] texCoordArray;

          delete [] triangleArray;
       }
  };



//______________________________________________________________________________ material Constructor

material::material(void) 
  {

    Gloss        = false ;
    EnvMap       = false ;

  }



//______________________________________________________________________________ Load Material Data Function

bool objFile::LoadMD( char *filename)

  //
  //  Loads *mdf files for texture paths and shaders used.
  //

  {

    char in_str[150];
    uint i , j;
    ifstream ifs_mat;


    ifs_mat.open(filename, ios::in);


    if(ifs_mat.fail())
      {
        ErrorLog( "Unable to load :" , filename );
        return false;
      }
    else 
      {
        ifs_mat.getline(in_str, 99, '\n');

        for( i=0; i<MaterialCount; i++ )
          {
            ifs_mat >> in_str
                    >> in_str
                    >> in_str 
                    >> in_str 
                    >> in_str 
                    >> in_str 
                    >> in_str
                    >> in_str 
                    >> in_str 
                    >> in_str 
                    >> in_str 
                    >> in_str
                    >> in_str 
                    >> in_str
                    >> in_str
                    >> in_str
                    >> in_str
                    >> in_str 
                    >> in_str
                    >> in_str
                    >> in_str
                    >> in_str
                    >> in_str
                    >> in_str;

            //Texture unit 1 - Functions & Texture Path...

            ifs_mat >> in_str 
                    >> in_str;
  
            for( j=0; j<128; j++ )
              {
                MaterialArray[ i ].TexPathA[ j ] = in_str[ j ];
              }

 
 
            ifs_mat >> in_str
                    >> in_str;
  

            ifs_mat >> in_str
                    >> in_str
                    >> in_str
                    >> in_str
                    >> in_str;

            //Texture unit 2 - Functions & Texture Path...

            ifs_mat >> in_str 
                    >> in_str;
 
            for( j = 0 ; j < 128 ; j++ )
              {
                MaterialArray[ i ].TexPathB[ j ] = in_str[ j ];
              }  


            ifs_mat >> in_str
                    >> in_str;
 
            ifs_mat >> in_str
                    >> in_str
                    >> in_str
                    >> in_str
                    >> in_str;

            //Shader Usage Booleans...

            ifs_mat >> in_str
                    >> in_str;

            ifs_mat >> in_str
                    >> in_str;

            ifs_mat >> in_str
                    >> in_str;


            if( strcmp( in_str,"TRUE" ) == 0 )
              {
                MaterialArray[i].Gloss = true;
              }
            else
              {
                MaterialArray[i].Gloss = false;
              }


            ifs_mat >> in_str
                    >> in_str;


            if( strcmp( in_str,"TRUE" ) == 0 )
              {
                MaterialArray[i].EnvMap = true;
              }
            else
              {
                MaterialArray[i].EnvMap = false;
              }


            //
            //  load textures...
            //

            if( MaterialArray[i].TexPathA[0] )
              {
                loadTexture(  MaterialArray[i].TexPathA ,
                             &MaterialArray[i].TexObjA  );
              }

            if( MaterialArray[i].TexPathB[0] )
              {
                loadTexture(  MaterialArray[i].TexPathB ,
                             &MaterialArray[i].TexObjB  );
              }

         } //end of for

       ifs_mat.close();

     }

   return true;

  }


