//
//  TITLE  : Simple Reflection Demo 
//
//  AUTHOR : Robert Bateman
//
//  DATE   : 8 March 2001
//
//  NOTES  : Uses a single plane to represent a mirror, and a cube is
//           reflected in realtime. All lighting is pre-calculated, and
//           recalculated when the object is moved. The code uses a
//           stencil buffer to mask out the region for drawing.
//


//______________________________________________________________ Includes

#include <iostream.h>
#include <fstream.h>
#include <malloc.h>
#include <string.h>

#include "./opengl_includes.h"

#ifdef WIN32
  #include <GL/glui.h>
#else
  #include "./glui.h"
#endif


// simple state definitions to be passed into GLUI callback func
#define OBJECT_MOVED 1
#define PLANE_MOVED  2
#define LIGHT_MOVED  3

int iModifier;

#include "./fruity_exception.h"
#include "./fruity_timer.h"
#include "./fruity_font.h" 
#include "./fruity_mayamouse.h"

#include "./objloader.h"
#include "./object_plane.h"
#include "./tga.h"
#include "./scene_lighting.h"

//______________________________________________________________ Globals

// Variables to deal with performance counters

float         fProcessingTime   = 0 ,
              fDisplayTime      = 0 ,
              fProcessingLast   = 0 ,
              fDisplayLast      = 0 ,
              fFrametime        = 0 ;
long          frames            = 0 ;
char          pzStatusLine[128]     ;


// 2D & 3D Views held in display lists

GLuint        Persp   ,
              Ortho   ,
              WoodTex ,
              EnvMap  ,
              MetalTex;

// The light structure used

cPointLight  pntL ,pntL2 ;

class cDynamicObject : public objFile
  {

     public :

       cDynamicObject(void) : objFile()
         {
             t    =  0;
             tx   = 13;
             ty   =  8;
             tz   =  0;
             ry   =  0;
             lrx  =  0;
             lry  =  0;
             lrz  =  0;
             drx  =  0;
             dry  =  0;
             drz  =  0;  
             drxs = (float)(4000 + rand()%10000)/10000;
             drys = (float)(4000 + rand()%10000)/10000;
             drzs = (float)(4000 + rand()%10000)/10000;
             glr  =  70;
   

         }

       float tx   ,
             ty   ,
             tz   ,
             ry   ,
             lrx  ,
             lry  ,
             lrz  ,
             drxs ,
             drys ,
             drzs ,
             drx  ,
             dry  ,
             drz  ,
             t    ,
             glr  ;

        void draw_shadows( void )
          {
            glBegin( GL_TRIANGLES );
            for( uint i=0; i<triangleCount; i++ )
              {
                 glVertex3v( vertexArray[ triangleArray[i].vinds[0] ].ShadowVec );
                 glVertex3v( vertexArray[ triangleArray[i].vinds[1] ].ShadowVec );
                 glVertex3v( vertexArray[ triangleArray[i].vinds[2] ].ShadowVec );
              }
            glEnd();
          }

        virtual void update( float tinc )
          {
             uint i;
             t+= tinc;
             drx = 0.5*(1 - sin(drxs*t));
             dry = 0.5*(1 - sin(drys*t));
             drz = 0.5*(1 - sin(drzs*t));
             lrx += drx;
             lry += dry;
             lrz += drz;
             ry  += glr*tinc;
             matrix m;
             m.rotateY( ry ).translate( tx,ty+sin(drys*t),tz ).rotateY( lry ).rotateX( lrx ).rotateZ( lrz );

             for( i=0; i<vertexCount;i++ )
               {
                  vertexArray[i].CurrentPos = m*vertexArray[i];
               }
             for( i=0; i<triangleCount; i++ )
               {
                  normalArray[ triangleArray[i].ninds[0] ].CurrentPos =
                     m*(normalArray[ triangleArray[i].ninds[0] ]+vertexArray[ triangleArray[i].vinds[0] ])
                     - vertexArray[ triangleArray[i].vinds[0] ].CurrentPos;
                  normalArray[ triangleArray[i].ninds[1] ].CurrentPos =
                     m*(normalArray[ triangleArray[i].ninds[1] ]+vertexArray[ triangleArray[i].vinds[1] ])
                     - vertexArray[ triangleArray[i].vinds[1] ].CurrentPos;
                  normalArray[ triangleArray[i].ninds[2] ].CurrentPos =
                     m*(normalArray[ triangleArray[i].ninds[2] ]+vertexArray[ triangleArray[i].vinds[2] ])
                     - vertexArray[ triangleArray[i].vinds[2] ].CurrentPos;
               }


             calc_lights( &pntL , false );
          }

   };



uint winHandle;

bool pointInPoly(const cVector3f& point, uint poly , objFile& obj )

// quick routine adapted from code by Ronald Frazier

{
	cVector3f N( 0 , 1 , 0 );

	unsigned int i,j;
	for (i=0,j=2; i<3; j=i++)
	{
		cVector3f T =  obj.vertexArray[ obj.triangleArray[poly].vinds[i] ].ShadowVec
                     - obj.vertexArray[ obj.triangleArray[poly].vinds[j] ].ShadowVec;
		cVector3f P = point - obj.vertexArray[ obj.triangleArray[poly].vinds[j] ].ShadowVec;

		if (((N ^ T) * P) < 0) return false;
	};

	return true;
};

bool pointInBbox(const cVector3f& point, objFile *obj )

// quick routine adapted from code by Ronald Frazier

{
	cVector3f N( 0 , 1 , 0 );
    cVector3f Vs[4];
    Vs[0] = obj->boundingBox[2];
    Vs[1].x = obj->boundingBox[3].x;
    Vs[1].z = obj->boundingBox[2].z;
    Vs[2]   = obj->boundingBox[3];
    Vs[3].x = obj->boundingBox[2].x;
    Vs[3].z = obj->boundingBox[3].z;

	unsigned int i,j;
	for (i=0,j=3; i<4; j=i++)
	{
		cVector3f T = Vs[i] - Vs[j];
		cVector3f P = point - Vs[j];

		if (((N ^ T) * P) < 0) return false;
	};

	return true;
};

void testForAndDrawShadowedPolys( cDynamicObject *dst , cDynamicObject  *src )
  {
     cVector3f p1( dst->boundingBox[2].x , 0 , dst->boundingBox[3].z ) ,
               p2( dst->boundingBox[3].x , 0 , dst->boundingBox[2].z ) ;
 uint i;
               // other two are dst.boundingBox[2] & dst.boundingBox[3]

     if( pointInBbox( dst->boundingBox[2] , src ) ||
         pointInBbox( dst->boundingBox[3] , src ) ||
         pointInBbox( p1 , src ) ||
         pointInBbox( p2 , src ) )
       {
          for( i=0; i< dst->vertexCount; i++ )
            {
              if( pointInBbox( dst->vertexArray[i].ShadowVec , src )) dst->vertexArray[i].inShadow = true;
            }
          glBegin( GL_TRIANGLES );
          for( i=0; i< dst->triangleCount; i++ )
            {
              if( dst->triangleArray[i].isLit )
              if( (dst->vertexArray[ dst->triangleArray[i].vinds[0] ].inShadow) ||
                  (dst->vertexArray[ dst->triangleArray[i].vinds[1] ].inShadow) ||
                  (dst->vertexArray[ dst->triangleArray[i].vinds[2] ].inShadow) )
                {
                  glVertex3v( dst->vertexArray[ dst->triangleArray[i].vinds[0] ].ShadowVec );
                  glVertex3v( dst->vertexArray[ dst->triangleArray[i].vinds[1] ].ShadowVec );
                  glVertex3v( dst->vertexArray[ dst->triangleArray[i].vinds[2] ].ShadowVec );
                }
            }
          glEnd();
       }
  }


bool drawShadowVols( objFile &obj , cLight *light )

  //
  //  Translates all verticies of specified objFile along a vector
  //  specified by the reletive position of light, until the points
  //  are all lying on the plane.
  //

  {
    if( obj.triangleCount == 0 ) return false;
 
    uint number_of_edges_for_shadow_poly = 0 , i;
    uint refs[ 80 ]; // allows up to twenty silhouette edges

    for( i=0; i<obj.triangleCount; i++ )
      {
          {
            if( obj.triangleArray[i].shadowEdgeMarkers[0] != -1 )
              {
                  // if passes this test, we have silhouette edge
                if( obj.triangleArray[i].isLit != 
                    obj.triangleArray[ obj.triangleArray[i].shadowEdgeMarkers[0] ].isLit )
                  {
                     refs[ 2*number_of_edges_for_shadow_poly    ] = obj.triangleArray[i].vinds[0];
                     refs[ 2*number_of_edges_for_shadow_poly +1 ] = obj.triangleArray[i].vinds[1];
                     number_of_edges_for_shadow_poly +=1 ;
                  }
              }
            if( obj.triangleArray[i].shadowEdgeMarkers[1] != -1 )
              {
                  // if passes this test, we have silhouette edge
                if( obj.triangleArray[i].isLit != 
                    obj.triangleArray[ obj.triangleArray[i].shadowEdgeMarkers[1] ].isLit )
                  {
                     refs[ 2*number_of_edges_for_shadow_poly    ] = obj.triangleArray[i].vinds[1];
                     refs[ 2*number_of_edges_for_shadow_poly +1 ] = obj.triangleArray[i].vinds[2];
                     number_of_edges_for_shadow_poly +=1 ;
                  }
              }
            if( obj.triangleArray[i].shadowEdgeMarkers[2] != -1 )
              {
                  // if passes this test, we have silhouette edge
                if( obj.triangleArray[i].isLit != 
                    obj.triangleArray[ obj.triangleArray[i].shadowEdgeMarkers[2] ].isLit )
                  {
                     refs[ 2*number_of_edges_for_shadow_poly    ] = obj.triangleArray[i].vinds[2];
                     refs[ 2*number_of_edges_for_shadow_poly +1 ] = obj.triangleArray[i].vinds[0];
                     number_of_edges_for_shadow_poly +=1 ;
                  }
              }
          }
      }

   glPushMatrix();
   glBegin( GL_QUADS );
     for( i=0; i<obj.triangleCount; i++ )
       {
          plane4f testP( obj.vertexArray[ obj.triangleArray[i].vinds[0] ].CurrentPos ,
                         obj.vertexArray[ obj.triangleArray[i].vinds[1] ].CurrentPos ,
                         obj.vertexArray[ obj.triangleArray[i].vinds[2] ].CurrentPos  );
         if( testP.distance( *light ) > 0 )
          {
          glVertex3v( obj.vertexArray[ obj.triangleArray[i].vinds[0] ].CurrentPos );
          glVertex3v( obj.vertexArray[ obj.triangleArray[i].vinds[1] ].CurrentPos );
          glVertex3v( obj.vertexArray[ obj.triangleArray[i].vinds[1] ].ShadowVec );
          glVertex3v( obj.vertexArray[ obj.triangleArray[i].vinds[0] ].ShadowVec );

          glVertex3v( obj.vertexArray[ obj.triangleArray[i].vinds[1] ].CurrentPos );
          glVertex3v( obj.vertexArray[ obj.triangleArray[i].vinds[2] ].CurrentPos );
          glVertex3v( obj.vertexArray[ obj.triangleArray[i].vinds[2] ].ShadowVec );
          glVertex3v( obj.vertexArray[ obj.triangleArray[i].vinds[1] ].ShadowVec );

          glVertex3v( obj.vertexArray[ obj.triangleArray[i].vinds[2] ].CurrentPos );
          glVertex3v( obj.vertexArray[ obj.triangleArray[i].vinds[0] ].CurrentPos );
          glVertex3v( obj.vertexArray[ obj.triangleArray[i].vinds[0] ].ShadowVec );
          glVertex3v( obj.vertexArray[ obj.triangleArray[i].vinds[2] ].ShadowVec );
          }
       }
    glEnd();
   glPopMatrix();

    return true;
  }


// The 3D cube

cDynamicObject cube , cube2,
               pyramid , pyramid2;


// the 3D reflection plane.

cObjectPlane plane ;



float pntL_pos_x = 0,
      pntL_pos_y = 25,
      pntL_pos_z = 0,
      pntL_area  = 1;

//______________________________________________________________ Function Declarations
 
void init( void );
void idle( void );
void display( void );
void reshape( int , int );
void Mouse_Motion( int , int );
void Mouse_Button( int , int , int , int );


//______________________________________________________________ Init Function


void init( void )

  { 
 
    InitFrameCounter();
    makeRasterFont();

    glClearColor( 0.0 , 0.0 , 0.0 , 0.0 );
    glClearStencil( 0x0 );
    glShadeModel( GL_SMOOTH );


      // set up depth testing
    glDepthFunc( GL_LESS );

    glBlendFunc( GL_SRC_ALPHA ,GL_ONE_MINUS_SRC_ALPHA );
          glDisable( GL_TEXTURE_2D );
      // load up geometry
    cube.load  ( "./data/scene_cube.obj"   );
    cube2.load( "./data/scene_cube.obj" );
    plane.load ( "./data/scene_floor.obj" );
    pyramid.load( "./data/scene_pyramid.obj" );
    pyramid2.load( "./data/scene_pyramid.obj" );

    pyramid.glr = -40;
    pyramid2.glr = -40;

    pyramid.ty = 18;
    pyramid2.ty = 18;

    pyramid.tx = 5;
    pyramid2.tx = 5;

      // load textures
    loadTexture( "./data/ceiling.tga" , &WoodTex );
    loadTexture( "./data/cube.tga"    , &MetalTex );
    loadTexture( "./data/scene_env.tga" , &EnvMap );

      // set up and pre-calculate lighting
    pntL.set( pntL_pos_x , pntL_pos_y , pntL_pos_z );

    pntL.pDiffuse[0] = 1.0;
    pntL.pDiffuse[1] = 1.0;
    pntL.pDiffuse[2] = 1.0;
    pntL.pAmbient[0] = 0.3;
    pntL.pAmbient[1] = 0.3;
    pntL.pAmbient[2] = 0.3;
    pntL.fLinearAttenuation    = 0.05 ;
    pntL.fQuadraticAttenuation = 0.002;


    // a copy of the first light is used to generate soft shadows.

    pntL2.pDiffuse[0] = 1;
    pntL2.pDiffuse[1] = 1;
    pntL2.pDiffuse[2] = 1.0;
    pntL2.pAmbient[0] = 0.3;
    pntL2.pAmbient[1] = 0.3;
    pntL2.pAmbient[2] = 0.3;
    pntL2.fLinearAttenuation    = 0.05 ;
    pntL2.fQuadraticAttenuation = 0.002;

    cube2.ry = 180;
    pyramid2.ry = 180;
    plane.calc_lights( &pntL , false );
    //while( cube2.ry == 180 );
  }



//______________________________________________________________ Idle Function

void idle( void )
  {
    frames++;


    fProcessingLast = GetFrameTime();
    fProcessingTime += fProcessingLast;
    fFrametime = divide_d( (fProcessingTime + fDisplayTime) , frames );

    glutSetWindow( winHandle );
    glutPostRedisplay();
  }




//______________________________________________________________ Display Function

void display( void )

  // 5 sections in total

  {
 
      // value is decremented if camera falls in a shadow volume
    uint stencilShadowReferenceValue = 128;

    glClear( GL_COLOR_BUFFER_BIT |
             GL_DEPTH_BUFFER_BIT |
             GL_STENCIL_BUFFER_BIT );

    glCallList( Persp );

      // perform matrix transformations on objects to get
      // all transformed vertex positions - need it for 
      // lighting calculation etc
    cube.update( fFrametime );
    cube2.update( fFrametime );
    pyramid.update( fFrametime );
    pyramid2.update( fFrametime );

      // calc chadow vectors when projected onto floor plane 
    plane.drawShadowOnPlane( cube , &pntL );
    plane.drawShadowOnPlane( cube2 ,&pntL );
    plane.drawShadowOnPlane( pyramid , &pntL );
    plane.drawShadowOnPlane( pyramid2 , &pntL );

    glPushMatrix();
      Mousey.doTransform();
      glPushMatrix();

        glEnable( GL_TEXTURE_2D );
        glEnable( GL_DEPTH_TEST );
        glDepthFunc( GL_LESS );


        glPushMatrix();
          glBindTexture( GL_TEXTURE_2D , WoodTex );
          plane.draw();
        glPopMatrix();

        glDisable( GL_TEXTURE_2D );
        glDisable( GL_DEPTH_TEST );
        glEnable( GL_BLEND );
        glBlendFunc( GL_SRC_ALPHA , GL_ONE_MINUS_SRC_ALPHA );
        glDisable( GL_CULL_FACE);

        glEnable( GL_STENCIL_TEST );
	    glStencilFunc(GL_EQUAL, 0x0, 0x1);
        glStencilOp(GL_KEEP, GL_KEEP, GL_INCR );

        glPushMatrix();
          glColor4f( 0.0 , 0.0 , 0.0 , 0.4 );
          pyramid.draw_shadows();
          pyramid2.draw_shadows();
          cube.draw_shadows();
          cube2.draw_shadows();
        glPopMatrix();


        glEnable( GL_CULL_FACE);
        glDisable( GL_BLEND );
        glEnable ( GL_TEXTURE_2D );
        glEnable ( GL_DEPTH_TEST );
        glDisable( GL_STENCIL_TEST );

          // draw cube objects with a plain texture on them.
          // this will give us depth buffering info....
          // lighting info is not taken into account on this pass,
          // so we are effectivly drawing the objects with global
          // ambient illumination
        glPushMatrix();
          cube.draw_no_col();
          cube2.draw_no_col();
        glPopMatrix();

        glClear( GL_STENCIL_BUFFER_BIT );
        glEnable( GL_STENCIL_TEST );
        glDisable( GL_TEXTURE_2D );

          // second pass, draw polys on cubes that may be in shadow
        glPushMatrix();
          glStencilFunc( GL_ALWAYS , 0x1 , 0x1 );
          glStencilOp  ( GL_REPLACE , GL_REPLACE , GL_REPLACE );
          glColorMask( 0,0,0,0 );
          glDepthMask( 0 );
          glDepthFunc( GL_EQUAL );
          testForAndDrawShadowedPolys( &cube  , &pyramid );
          testForAndDrawShadowedPolys( &cube2 , &pyramid );
        glPopMatrix();

        glDisable( GL_CULL_FACE );

          // draw shadow volumes,
          // this uses the depth buffering info from the first pass to
          // determine if the shadow volume appears infront of an object
          // - if it does 
        glPushMatrix();

	      glStencilFunc(GL_EQUAL, 0x1, 0x2);
          glStencilOp(GL_KEEP, GL_KEEP, GL_INCR );
          glDepthFunc( GL_LESS );
          drawShadowVols( pyramid  , &pntL );
          drawShadowVols( pyramid2 , &pntL );
        glPopMatrix();

        glEnable( GL_CULL_FACE );

      glPopMatrix();

	  //re-enable depth and color buffer writes
	  glDepthMask(1);
	  glColorMask(1,1,1,1);

	  //re-enable texturing
	  glEnable(GL_TEXTURE_2D);
	  glDepthFunc( GL_EQUAL );
      glPushMatrix();

	    //prepare the stencil buffer so that we only draw to unshadowed areas
	    //(unshadowed means that the stencil buffer value equals the reference value)
	    glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
	    glStencilFunc( GL_GREATER, 0x1, 0xff);

        glPushMatrix();
          glBindTexture( GL_TEXTURE_2D , WoodTex );
          cube.draw();
          cube2.draw();
        glPopMatrix();


        glDisable( GL_STENCIL_TEST );

        glPushMatrix();
     	  glDepthFunc( GL_LEQUAL );

          glBindTexture( GL_TEXTURE_2D , MetalTex );
          pyramid.draw();
          pyramid2.draw();

          glDisable(GL_TEXTURE_2D);

        glPopMatrix();

        glPushMatrix();
	      glDepthFunc( GL_LESS );
          glColor3f( 1,1,0);
          glDisable( GL_TEXTURE_2D );
          glTranslatev( pntL );
          glutSolidCube( 0.1 );
        glPopMatrix(); 

      glPopMatrix();
    glPopMatrix();

 
    // Draw Performance Counters on a 2D overlay

    glCallList( Ortho );

    sprintf( pzStatusLine , 
            "Average Processing : %f secs : Last Frame : %f" ,
             divide_d( fProcessingTime , frames ) ,
             fProcessingLast                    );
    font(    pzStatusLine  , 5 , 12 , 1, 1, 0.4 );


    sprintf( pzStatusLine ,
             "Average Display Time : %f secs : Last Frame : %f" ,
             divide_d( fDisplayTime , frames   ) ,
             fDisplayLast                      ) ;
    font(    pzStatusLine , 5 , 22 , 1, 1, 0.4 ) ;


    sprintf( pzStatusLine ,
             "Average fps: %f : Last Frame : %f" ,
             divide_d( 1 , fFrametime) ,
             divide_f( 1 , (fDisplayLast+fProcessingLast) ) );
    font(    pzStatusLine , 5 , 32 , 1, 1, 1 );



    glutSwapBuffers();


    fDisplayLast  = GetFrameTime();;
    fDisplayTime += fDisplayLast;

  }



//______________________________________________________________ Reshape Function

void reshape( int w , int h )
  
  //
  //  The reshape function resets performance counters and
  //  resizes the window.
  //


  {
   
    glDeleteLists(Persp,1);
    glDeleteLists(Ortho,1);
    Persp = glGenLists(1);
    Ortho = glGenLists(1);

    fProcessingTime = 0;
    fDisplayTime    = 0;
    fProcessingLast = 0; 
    fDisplayLast    = 0;
    fFrametime      = 0;
    frames          = 0;

    glNewList(Persp,GL_COMPILE);
      glMatrixMode( GL_TEXTURE );
      glLoadIdentity();
      glViewport(0, (GLsizei) 0, (GLsizei) w, (GLsizei) h);
      glMatrixMode(GL_PROJECTION);
      glLoadIdentity();
      gluPerspective(55.0, divide_f( w , h ) , 0.01, 500 );
      glMatrixMode(GL_MODELVIEW);
      glLoadIdentity();
    glEndList();

    glNewList(Ortho,GL_COMPILE);
      glViewport(0, (GLsizei) 0, (GLsizei) w, (GLsizei) h);
      glMatrixMode(GL_PROJECTION);
      glLoadIdentity();
      glOrtho(0, w, h, 0, -1, 1);
      glMatrixMode(GL_MODELVIEW);
      glLoadIdentity();
      glDisable( GL_TEXTURE_2D );
      glDisable( GL_DEPTH_TEST );
    glEndList();

    glCallList(Persp);
    GetFrameTime();
  }

 
//______________________________________________________________ Mouse Motion Function

void Mouse_Motion(int x, int y)

  //
  //  This function deals with mouse movement events & passes values to
  //  the class defined in "./mayamouse.h" which sorts out maya style input...
  //

  {
    Mousey.motion( x , y );




    glutPostRedisplay();
  }


//______________________________________________________________ Mouse Button Function

void Mouse_Button( int button , int state , int x , int y )

  //
  //  This function deals with mouse button events & passes values to
  //  the class defined in "./mayamouse.h" which sorts out maya style input...
  //
 
  {
    Mousey.buttonPress( button , state , x , y );

    //if

    glutPostRedisplay();
  }


//______________________________________________________________ Main Function

int main( int argc , char **argv )

  {

     glutInit(&argc, argv);
     glutInitDisplayMode   ( GLUT_DOUBLE |
                             GLUT_RGBA   |
                             GLUT_DEPTH  |
                             GLUT_STENCIL );
     glutInitWindowSize    ( 800 , 600 );
     glutInitWindowPosition( 100 , 100 );
     winHandle = glutCreateWindow      ( "Realtime Soft Edged Shadow Demo - LMB controls cube, RMB/MMB controls floor, hold ALT to move around view" );

     glutDisplayFunc( display );
     glutReshapeFunc( reshape );

     init();
  


     glutMotionFunc ( Mouse_Motion );

     glutMouseFunc   ( Mouse_Button );
     glutIdleFunc    ( idle );

     glutMainLoop();
     return 0;

  }


