/*#####################################################################################
 *
 *   Title        : fruity_glext.cpp
 *   Author       : Robert Bateman
 *   Date         : 15-Sep-2000
 *   Last Updated : 30-Nov-2000
 *
#####################################################################################*/

/*-------------------------------------------------------------------------------------

A BRIEF EXPLANATION : 


The standard "Glext.h" file contains definitions of external functions. When you include
the file in your programs, it lets the compiler know that the functions exist somewhere
in the body of your code.

This leads to a wee bit of a problem. The functions you wish to use will have to be 
defined somewhere as a normal function rather than just an external function.

This is where Windows implimentation of openGL extensions is a bit problematic. We
need to declare the function as a pointer. This pointer needs to be set at runtime
to point to the actual part of the graphics card driver that deals with this function.

The steps are:

1. Defining the function pointer

                    PFNGLBLENDCOLORPROC  glBlendColor = NULL;

   The first part is a pointer to a function of type 'glBlendColor'. We then give it
   a name, in this case 'glBlendColor' which relates nicely to the extension definition.
   We then set this pointer to NULL.

2. Getting the address at runtime

         glBlendColor = (PFNGLBLENDCOLORPROC)  wglGetProcAddress("glBlendColor");

   "glBlendColor = " - This assigns a new memory adress for function pointer 'glBlendColor'
                       to point to. We want this to be the address of the related function.
   "wglGetProcAddress("glBlendColor")" - This returns The adress of the function 'glBlendColor'
                                         from the openGL drivers.
   "(PFNGLBLENDCOLORPROC)" - wglGetProcAddress returns type 'void *' so it needs to be
                             casted to the appropriate function pointer type.

Once these stages are conmplete, the function maybe used as normal.

-------------------------------------------------------------------------------------*/


#define MAX_EXTENSION_SPACE 10240   // Characters for Extension-Strings
#define MAX_EXTENSION_LENGTH 256    // Maximum Of Characters In One Extension-String


#include <string.h>

#include "./fruity_glext.h"

GLboolean isInString   ( char * , const char * );
void      initExtension( char * );
GLboolean CheckGLExt( char *);

//#####################################################################################
//                                   OPENGL 1.2
//#####################################################################################

#ifdef GL_VERSION_1_2
  PFNGLBLENDCOLORPROC                glBlendColor                = NULL;
  PFNGLBLENDEQUATIONPROC             glBlendEquation             = NULL;
  PFNGLDRAWRANGEELEMENTSPROC         glDrawRangeElements         = NULL;
  PFNGLCOLORTABLEPROC                glColorTable                = NULL;
  PFNGLCOLORTABLEPARAMETERFVPROC     glColorTableParameterfv     = NULL;
  PFNGLCOLORTABLEPARAMETERIVPROC     glColorTableParameteriv     = NULL;
  PFNGLCOPYCOLORTABLEPROC            glCopyColorTable            = NULL;
  PFNGLGETCOLORTABLEPROC             glGetColorTable             = NULL;
  PFNGLGETCOLORTABLEPARAMETERFVPROC  glGetColorTableParameterfv  = NULL;
  PFNGLGETCOLORTABLEPARAMETERIVPROC  glGetColorTableParameteriv  = NULL;
  PFNGLCOLORSUBTABLEPROC             glColorSubTable             = NULL;
  PFNGLCOPYCOLORSUBTABLEPROC         glCopyColorSubTable         = NULL;
  PFNGLCONVOLUTIONFILTER1DPROC       glConvolutionFilter1D       = NULL;
  PFNGLCONVOLUTIONFILTER2DPROC       glConvolutionFilter2D       = NULL;
  PFNGLCONVOLUTIONPARAMETERFPROC     glConvolutionParameterf     = NULL;
  PFNGLCONVOLUTIONPARAMETERFVPROC    glConvolutionParameterfv    = NULL;
  PFNGLCONVOLUTIONPARAMETERIPROC     glConvolutionParameteri     = NULL;
  PFNGLCONVOLUTIONPARAMETERIVPROC    glConvolutionParameteriv    = NULL;
  PFNGLCOPYCONVOLUTIONFILTER1DPROC   glCopyConvolutionFilter1D   = NULL;
  PFNGLCOPYCONVOLUTIONFILTER2DPROC   glCopyConvolutionFilter2D   = NULL;
  PFNGLGETCONVOLUTIONFILTERPROC      glGetConvolutionFilter      = NULL;
  PFNGLGETCONVOLUTIONPARAMETERFVPROC glGetConvolutionParameterfv = NULL;
  PFNGLGETCONVOLUTIONPARAMETERIVPROC glGetConvolutionParameteriv = NULL;
  PFNGLGETSEPARABLEFILTERPROC        glGetSeparableFilter        = NULL;
  PFNGLSEPARABLEFILTER2DPROC         glSeparableFilter2D         = NULL;
  PFNGLGETHISTOGRAMPROC              glGetHistogram              = NULL;
  PFNGLGETHISTOGRAMPARAMETERFVPROC   glGetHistogramParameterfv   = NULL;
  PFNGLGETHISTOGRAMPARAMETERIVPROC   glGetHistogramParameteriv   = NULL;
  PFNGLGETMINMAXPROC                 glGetMinmax                 = NULL;
  PFNGLGETMINMAXPARAMETERFVPROC      glGetMinmaxParameterfv      = NULL; 
  PFNGLGETMINMAXPARAMETERIVPROC      glGetMinmaxParameteriv      = NULL;
  PFNGLHISTOGRAMPROC                 glHistogram                 = NULL;
  PFNGLMINMAXPROC                    glMinmax                    = NULL;
  PFNGLRESETHISTOGRAMPROC            glResetHistogram            = NULL; 
  PFNGLRESETMINMAXPROC               glResetMinmax               = NULL;
  PFNGLTEXIMAGE3DPROC                glTexImage3D                = NULL;
  PFNGLTEXSUBIMAGE3DPROC             glTexSubImage3D             = NULL;
  PFNGLCOPYTEXSUBIMAGE3DPROC         glCopyTexSubImage3D         = NULL;
#endif


//#####################################################################################
//                               ARB MULTITEXTURE
//#####################################################################################

#ifdef GL_ARB_multitexture
  PFNGLACTIVETEXTUREARBPROC       glActiveTextureARB        = NULL;
  PFNGLCLIENTACTIVETEXTUREARBPROC glClientActiveTextureARB  = NULL;
  PFNGLMULTITEXCOORD1DARBPROC     glMultiTexCoord1dARB      = NULL;
  PFNGLMULTITEXCOORD1DVARBPROC    glMultiTexCoord1dvARB     = NULL;
  PFNGLMULTITEXCOORD1FARBPROC     glMultiTexCoord1fARB      = NULL;
  PFNGLMULTITEXCOORD1FVARBPROC    glMultiTexCoord1fvARB     = NULL;
  PFNGLMULTITEXCOORD1IARBPROC     glMultiTexCoord1iARB      = NULL;
  PFNGLMULTITEXCOORD1IVARBPROC    glMultiTexCoord1ivARB     = NULL;
  PFNGLMULTITEXCOORD1SARBPROC     glMultiTexCoord1sARB      = NULL;
  PFNGLMULTITEXCOORD1SVARBPROC    glMultiTexCoord1svARB     = NULL;
  PFNGLMULTITEXCOORD2DARBPROC     glMultiTexCoord2dARB      = NULL;
  PFNGLMULTITEXCOORD2DVARBPROC    glMultiTexCoord2dvARB     = NULL;
  PFNGLMULTITEXCOORD2FARBPROC     glMultiTexCoord2fARB      = NULL;
  PFNGLMULTITEXCOORD2FVARBPROC    glMultiTexCoord2fvARB     = NULL;
  PFNGLMULTITEXCOORD2IARBPROC     glMultiTexCoord2iARB      = NULL;
  PFNGLMULTITEXCOORD2IVARBPROC    glMultiTexCoord2ivARB     = NULL;
  PFNGLMULTITEXCOORD2SARBPROC     glMultiTexCoord2sARB      = NULL;
  PFNGLMULTITEXCOORD2SVARBPROC    glMultiTexCoord2svARB     = NULL;
  PFNGLMULTITEXCOORD3DARBPROC     glMultiTexCoord3dARB      = NULL;
  PFNGLMULTITEXCOORD3DVARBPROC    glMultiTexCoord3dvARB     = NULL;
  PFNGLMULTITEXCOORD3FARBPROC     glMultiTexCoord3fARB      = NULL;
  PFNGLMULTITEXCOORD3FVARBPROC    glMultiTexCoord3fvARB     = NULL;
  PFNGLMULTITEXCOORD3IARBPROC     glMultiTexCoord3iARB      = NULL;
  PFNGLMULTITEXCOORD3IVARBPROC    glMultiTexCoord3ivARB     = NULL;
  PFNGLMULTITEXCOORD3SARBPROC     glMultiTexCoord3sARB      = NULL;
  PFNGLMULTITEXCOORD3SVARBPROC    glMultiTexCoord3svARB     = NULL;
  PFNGLMULTITEXCOORD4DARBPROC     glMultiTexCoord4dARB      = NULL;
  PFNGLMULTITEXCOORD4DVARBPROC    glMultiTexCoord4dvARB     = NULL;
  PFNGLMULTITEXCOORD4FARBPROC     glMultiTexCoord4fARB      = NULL;
  PFNGLMULTITEXCOORD4FVARBPROC    glMultiTexCoord4fvARB     = NULL;
  PFNGLMULTITEXCOORD4IARBPROC     glMultiTexCoord4iARB      = NULL;
  PFNGLMULTITEXCOORD4IVARBPROC    glMultiTexCoord4ivARB     = NULL;
  PFNGLMULTITEXCOORD4SARBPROC     glMultiTexCoord4sARB      = NULL;
  PFNGLMULTITEXCOORD4SVARBPROC    glMultiTexCoord4svARB     = NULL;
#endif

//#####################################################################################
//                              ARB TRANSPOSE MATRIX
//#####################################################################################

#ifdef GL_ARB_transpose_matrix
  PFNGLLOADTRANSPOSEMATRIXFARBPROC glLoadTransposeMatrixfARB = NULL;
  PFNGLLOADTRANSPOSEMATRIXDARBPROC glLoadTransposeMatrixdARB = NULL;
  PFNGLMULTTRANSPOSEMATRIXFARBPROC glMultTransposeMatrixfARB = NULL;
  PFNGLMULTTRANSPOSEMATRIXDARBPROC glMultTransposeMatrixdARB = NULL;
#endif

//#####################################################################################
//                                ARB MULTISAMPLE
//#####################################################################################

#ifdef GL_ARB_multisample
  PFNGLSAMPLECOVERAGEARBPROC  glSampleCoverageARB = NULL;
  PFNGLSAMPLEPASSARBPROC      glSamplePassARB     = NULL;
#endif

//#####################################################################################
//                             ARB TEXTURE COMPRESSION
//#####################################################################################

#ifdef GL_ARB_texture_compression
  PFNGLCOMPRESSEDTEXIMAGE3DARBPROC      glCompressedTexImage3DARB     = NULL;
  PFNGLCOMPRESSEDTEXIMAGE2DARBPROC      glCompressedTexImage2DARB     = NULL;
  PFNGLCOMPRESSEDTEXIMAGE1DARBPROC      glCompressedTexImage1DARB     = NULL;
  PFNGLCOMPRESSEDTEXSUBIMAGE3DARBPROC   glCompressedTexSubImage3DARB  = NULL;
  PFNGLCOMPRESSEDTEXSUBIMAGE2DARBPROC   glCompressedTexSubImage2DARB  = NULL;
  PFNGLCOMPRESSEDTEXSUBIMAGE1DARBPROC   glCompressedTexSubImage1DARB  = NULL;
  PFNGLGETCOMPRESSEDTEXIMAGEARBPROC     glGetCompressedTexImageARB    = NULL;
#endif

//#####################################################################################
//                      EXT BLEND COLOUR - Part Of OpenGL1.2
//#####################################################################################

#ifdef GL_EXT_blend_color
  PFNGLBLENDCOLOREXTPROC  glBlendColorEXT  = NULL;
#endif

//#####################################################################################
//                      EXT POLYGON OFFSET - Part Of OpenGL1.1
//#####################################################################################

#ifdef GL_EXT_polygon_offset
  PFNGLPOLYGONOFFSETEXTPROC glPolygonOffsetEXT = NULL;
#endif

//#####################################################################################
//                        EXT TEXTURE 3D - Part Of OpenGL1.2
//#####################################################################################

#ifdef GL_EXT_texture3D
  PFNGLTEXIMAGE3DEXTPROC     glTexImage3DEXT    = NULL;
  PFNGLTEXSUBIMAGE3DEXTPROC  glTexSubImage3DEXT = NULL;
#endif

//#####################################################################################
//                         EXT HISTOGRAM - Part Of OpenGL1.2
//#####################################################################################

#ifdef GL_EXT_histogram
  PFNGLGETHISTOGRAMEXTPROC             glGetHistogramEXT             = NULL;
  PFNGLGETHISTOGRAMPARAMETERFVEXTPROC  glGetHistogramParameterfvEXT  = NULL;
  PFNGLGETHISTOGRAMPARAMETERIVEXTPROC  glGetHistogramParameterivEXT  = NULL;
  PFNGLGETMINMAXEXTPROC                glGetMinmaxEXT                = NULL;
  PFNGLGETMINMAXPARAMETERFVEXTPROC     glGetMinmaxParameterfvEXT     = NULL;
  PFNGLGETMINMAXPARAMETERIVEXTPROC     glGetMinmaxParameterivEXT     = NULL;
  PFNGLHISTOGRAMEXTPROC                glHistogramEXT                = NULL;
  PFNGLMINMAXEXTPROC                   glMinmaxEXT                   = NULL;
  PFNGLRESETHISTOGRAMEXTPROC           glResetHistogramEXT           = NULL;
  PFNGLRESETMINMAXEXTPROC              glResetMinmaxEXT              = NULL;
#endif

//#####################################################################################
//                     EXT CONVOLUTION FILTERS - Part Of OpenGL1.2
//#####################################################################################

#ifdef GL_EXT_convolution
  PFNGLCONVOLUTIONFILTER1DEXTPROC       glConvolutionFilter1DEXT       = NULL;
  PFNGLCONVOLUTIONFILTER2DEXTPROC       glConvolutionFilter2DEXT       = NULL;
  PFNGLCONVOLUTIONPARAMETERFEXTPROC     glConvolutionParameterfEXT     = NULL;
  PFNGLCONVOLUTIONPARAMETERFVEXTPROC    glConvolutionParameterfvEXT    = NULL;
  PFNGLCONVOLUTIONPARAMETERIEXTPROC     glConvolutionParameteriEXT     = NULL;
  PFNGLCONVOLUTIONPARAMETERIVEXTPROC    glConvolutionParameterivEXT    = NULL;
  PFNGLCOPYCONVOLUTIONFILTER1DEXTPROC   glCopyConvolutionFilter1DEXT   = NULL;
  PFNGLCOPYCONVOLUTIONFILTER2DEXTPROC   glCopyConvolutionFilter2DEXT   = NULL;
  PFNGLGETCONVOLUTIONFILTEREXTPROC      glGetConvolutionFilterEXT      = NULL;
  PFNGLGETCONVOLUTIONPARAMETERFVEXTPROC glGetConvolutionParameterfvEXT = NULL;
  PFNGLGETCONVOLUTIONPARAMETERIVEXTPROC glGetConvolutionParameterivEXT = NULL;
  PFNGLGETSEPARABLEFILTEREXTPROC        glGetSeparableFilterEXT        = NULL;
  PFNGLSEPARABLEFILTER2DEXTPROC         glSeparableFilter2DEXT         = NULL;
#endif

//#####################################################################################
//                      EXT TEXTURE OBJECT - Part Of OpenGL1.1
//#####################################################################################

#ifndef GL_EXT_texture_object
  PFNGLARETEXTURESRESIDENTEXTPROC glAreTexturesResidentEXT = NULL;
  PFNGLBINDTEXTUREEXTPROC         glBindTextureEXT         = NULL;
  PFNGLDELETETEXTURESEXTPROC      glDeleteTexturesEXT      = NULL;
  PFNGLGENTEXTURESEXTPROC         glGenTexturesEXT         = NULL;
  PFNGLISTEXTUREEXTPROC           glIsTextureEXT           = NULL;
  PFNGLPRIORITIZETEXTURESEXTPROC  glPrioritizeTexturesEXT  = NULL;
#endif

//#####################################################################################
//                      EXT VERTEX ARRAY - Part Of OpenGL1.1
//#####################################################################################

#ifdef GL_EXT_vertex_array
  PFNGLARRAYELEMENTEXTPROC     glArrayElementEXT     = NULL;
  PFNGLCOLORPOINTEREXTPROC     glColorPointerEXT     = NULL;
  PFNGLDRAWARRAYSEXTPROC       glDrawArraysEXT       = NULL;
  PFNGLEDGEFLAGPOINTEREXTPROC  glEdgeFlagPointerEXT  = NULL;
  PFNGLGETPOINTERVEXTPROC      glGetPointervEXT      = NULL;
  PFNGLINDEXPOINTEREXTPROC     glIndexPointerEXT     = NULL;
  PFNGLNORMALPOINTEREXTPROC    glNormalPointerEXT    = NULL;
  PFNGLTEXCOORDPOINTEREXTPROC  glTexCoordPointerEXT  = NULL;
  PFNGLVERTEXPOINTEREXTPROC    glVertexPointerEXT    = NULL;
#endif

//#####################################################################################
//                                 EXT BLEND MINMAX
//#####################################################################################

#ifdef GL_EXT_blend_minmax
  PFNGLBLENDEQUATIONEXTPROC   glBlendEquationEXT  = NULL;
#endif

//#####################################################################################
//                               EXT POINT PARAMETERS
//#####################################################################################

#ifdef GL_EXT_point_parameters
  PFNGLPOINTPARAMETERFEXTPROC   glPointParameterfEXT   = NULL;
  PFNGLPOINTPARAMETERFVEXTPROC  glPointParameterfvEXT  = NULL;
  PFNGLPOINTPARAMETERFSGISPROC  glPointParameterfSGIS  = NULL;
  PFNGLPOINTPARAMETERFVSGISPROC glPointParameterfvSGIS = NULL;
#endif

//#####################################################################################
//                               EXT LIGHT TEXTURE
//#####################################################################################

#ifdef GL_EXT_light_texture
  PFNGLAPPLYTEXTUREEXTPROC    glApplyTextureEXT    = NULL;
  PFNGLTEXTURELIGHTEXTPROC    glTextureLightEXT    = NULL;
  PFNGLTEXTUREMATERIALEXTPROC glTextureMaterialEXT = NULL;
#endif

//#####################################################################################
//                                EXT INDEX MATERIAL
//#####################################################################################

#ifdef GL_EXT_index_material
  PFNGLINDEXMATERIALEXTPROC glIndexMaterialEXT = NULL;
#endif

//#####################################################################################
//                                  EXT INDEX FUNC
//#####################################################################################

#ifdef GL_EXT_index_func
  PFNGLINDEXFUNCEXTPROC glIndexFuncEXT = NULL;
#endif

//#####################################################################################
//                             EXT COMPILED VERTEX ARRAY
//#####################################################################################

#ifdef GL_EXT_compiled_vertex_array
  PFNGLLOCKARRAYSEXTPROC   glLockArraysEXT   = NULL;
  PFNGLUNLOCKARRAYSEXTPROC glUnlockArraysEXT = NULL;
#endif

//#####################################################################################
//                                EXT CULL VERTEX
//#####################################################################################

#ifdef GL_EXT_cull_vertex
  PFNGLCULLPARAMETERDVEXTPROC glCullParameterdvEXT = NULL;
  PFNGLCULLPARAMETERFVEXTPROC glCullParameterfvEXT = NULL;
#endif

//#####################################################################################
//                               EXT DRAW ARRAY RANGE
//#####################################################################################

#ifdef GL_EXT_draw_range_elements
  PFNGLDRAWRANGEELEMENTSEXTPROC glDrawRangeElementsEXT = NULL;
#endif

//#####################################################################################
//                                EXT PIXEL TRANSFORM
//#####################################################################################

#ifdef GL_EXT_pixel_transform
  PFNGLPIXELTRANSFORMPARAMETERIEXTPROC   glPixelTransformParameteriEXT  = NULL;
  PFNGLPIXELTRANSFORMPARAMETERFEXTPROC   glPixelTransformParameterfEXT  = NULL;
  PFNGLPIXELTRANSFORMPARAMETERIVEXTPROC  glPixelTransformParameterivEXT = NULL;
  PFNGLPIXELTRANSFORMPARAMETERFVEXTPROC  glPixelTransformParameterfvEXT = NULL;
#endif

//#####################################################################################
//                              EXT SECONDARY COLOUR
//#####################################################################################

#ifdef GL_EXT_secondary_color
  PFNGLSECONDARYCOLOR3BEXTPROC       glSecondaryColor3bEXT      = NULL;
  PFNGLSECONDARYCOLOR3BVEXTPROC      glSecondaryColor3bvEXT     = NULL;
  PFNGLSECONDARYCOLOR3DEXTPROC       glSecondaryColor3dEXT      = NULL;
  PFNGLSECONDARYCOLOR3DVEXTPROC      glSecondaryColor3dvEXT     = NULL;
  PFNGLSECONDARYCOLOR3FEXTPROC       glSecondaryColor3fEXT      = NULL;
  PFNGLSECONDARYCOLOR3FVEXTPROC      glSecondaryColor3fvEXT     = NULL;
  PFNGLSECONDARYCOLOR3IEXTPROC       glSecondaryColor3iEXT      = NULL;
  PFNGLSECONDARYCOLOR3IVEXTPROC      glSecondaryColor3ivEXT     = NULL;
  PFNGLSECONDARYCOLOR3SEXTPROC       glSecondaryColor3sEXT      = NULL;
  PFNGLSECONDARYCOLOR3SVEXTPROC      glSecondaryColor3svEXT     = NULL;
  PFNGLSECONDARYCOLOR3UBEXTPROC      glSecondaryColor3ubEXT     = NULL;
  PFNGLSECONDARYCOLOR3UBVEXTPROC     glSecondaryColor3ubvEXT    = NULL;
  PFNGLSECONDARYCOLOR3UIEXTPROC      glSecondaryColor3uiEXT     = NULL;
  PFNGLSECONDARYCOLOR3UIVEXTPROC     glSecondaryColor3uivEXT    = NULL;
  PFNGLSECONDARYCOLOR3USEXTPROC      glSecondaryColor3usEXT     = NULL;
  PFNGLSECONDARYCOLOR3USVEXTPROC     glSecondaryColor3usvEXT    = NULL;
  PFNGLSECONDARYCOLORPOINTEREXTPROC  glSecondaryColorPointerEXT = NULL;
#endif

//#####################################################################################
//                                EXT PERTURB NORMAL
//#####################################################################################

#ifdef GL_EXT_texture_perturb_normal
  PFNGLTEXTURENORMALEXTPROC  glTextureNormalEXT  = NULL;
#endif

//#####################################################################################
//                                  EXT FOG COORD
//#####################################################################################

#ifdef GL_EXT_fog_coord
  PFNGLFOGCOORDFEXTPROC       glFogCoordfEXT       = NULL;
  PFNGLFOGCOORDFVEXTPROC      glFogCoordfvEXT      = NULL;
  PFNGLFOGCOORDDEXTPROC       glFogCoorddEXT       = NULL;
  PFNGLFOGCOORDDVEXTPROC      glFogCoorddvEXT      = NULL;
  PFNGLFOGCOORDPOINTEREXTPROC glFogCoordPointerEXT = NULL;
#endif

//#####################################################################################
//                              EXT BLEND FUNC SEPERATE
//#####################################################################################

#ifdef GL_EXT_blend_func_separate
  PFNGLBLENDFUNCSEPARATEEXTPROC glBlendFuncSeparateEXT = NULL;
#endif

//#####################################################################################
//                                EXT VERTEX WEIGHTING
//#####################################################################################

#ifdef GL_EXT_vertex_weighting
  PFNGLVERTEXWEIGHTFEXTPROC       glVertexWeightfEXT       = NULL;
  PFNGLVERTEXWEIGHTFVEXTPROC      glVertexWeightfvEXT      = NULL;
  PFNGLVERTEXWEIGHTPOINTEREXTPROC glVertexWeightPointerEXT = NULL;
#endif

//#####################################################################################
//                               NV VERTEX ARRAY RANGE
//#####################################################################################

#ifdef GL_NV_vertex_array_range
  PFNGLFLUSHVERTEXARRAYRANGENVPROC glFlushVertexArrayRangeNV = NULL;
  PFNGLVERTEXARRAYRANGENVPROC      glVertexArrayRangeNV      = NULL;
#endif


//#####################################################################################
//                               NV REGISTER COMBINERS
//#####################################################################################

#ifdef GL_NV_register_combiners
  PFNGLCOMBINERPARAMETERFVNVPROC               glCombinerParameterfvNV              = NULL;
  PFNGLCOMBINERPARAMETERFNVPROC                glCombinerParameterfNV               = NULL;
  PFNGLCOMBINERPARAMETERIVNVPROC               glCombinerParameterivNV              = NULL;
  PFNGLCOMBINERPARAMETERINVPROC                glCombinerParameteriNV               = NULL;
  PFNGLCOMBINERINPUTNVPROC                     glCombinerInputNV                    = NULL;
  PFNGLCOMBINEROUTPUTNVPROC                    glCombinerOutputNV                   = NULL;
  PFNGLFINALCOMBINERINPUTNVPROC                glFinalCombinerInputNV               = NULL;
  PFNGLGETCOMBINERINPUTPARAMETERFVNVPROC       glGetCombinerInputParameterfvNV      = NULL;
  PFNGLGETCOMBINERINPUTPARAMETERIVNVPROC       glGetCombinerInputParameterivNV      = NULL;
  PFNGLGETCOMBINEROUTPUTPARAMETERFVNVPROC      glGetCombinerOutputParameterfvNV     = NULL;
  PFNGLGETCOMBINEROUTPUTPARAMETERIVNVPROC      glGetCombinerOutputParameterivNV     = NULL;
  PFNGLGETFINALCOMBINERINPUTPARAMETERFVNVPROC  glGetFinalCombinerInputParameterfvNV = NULL;
  PFNGLGETFINALCOMBINERINPUTPARAMETERIVNVPROC  glGetFinalCombinerInputParameterivNV = NULL;
#endif

//#####################################################################################
//                                EXT MULTISAMPLE
//#####################################################################################

#ifdef GL_EXT_multisample
  PFNGLSAMPLEMASKEXTPROC    glSampleMaskEXT    = NULL;
  PFNGLSAMPLEPATTERNEXTPROC glSamplePatternEXT = NULL;
#endif

//#####################################################################################
//                                   OPENGL 1.2
//#####################################################################################

GLboolean initGL_1_2(void) 
  {
    char *Version = (char *)glGetString(GL_VERSION);

    if(strncmp(Version,"1.2",3) == 0)
      {
        #ifdef GL_VERSION_1_2
          glBlendColor                = (PFNGLBLENDCOLORPROC)                wglGetProcAddress("glBlendColor");
          glBlendEquation             = (PFNGLBLENDEQUATIONPROC)             wglGetProcAddress("glBlendEquation"); 
          glDrawRangeElements         = (PFNGLDRAWRANGEELEMENTSPROC)         wglGetProcAddress("glDrawRangeElements"); 
          glColorTable                = (PFNGLCOLORTABLEPROC)                wglGetProcAddress("glColorTable");
          glColorTableParameterfv     = (PFNGLCOLORTABLEPARAMETERFVPROC)     wglGetProcAddress("glColorTableParameterfv"); 
          glColorTableParameteriv     = (PFNGLCOLORTABLEPARAMETERIVPROC)     wglGetProcAddress("glColorTableParameteriv");
          glCopyColorTable            = (PFNGLCOPYCOLORTABLEPROC)            wglGetProcAddress("glCopyColorTable");
          glGetColorTable             = (PFNGLGETCOLORTABLEPROC)             wglGetProcAddress("glGetColorTable");
          glGetColorTableParameterfv  = (PFNGLGETCOLORTABLEPARAMETERFVPROC)  wglGetProcAddress("glGetColorTableParameterfv");
          glGetColorTableParameteriv  = (PFNGLGETCOLORTABLEPARAMETERIVPROC)  wglGetProcAddress("glGetColorTableParameteriv");
          glColorSubTable             = (PFNGLCOLORSUBTABLEPROC)             wglGetProcAddress("glColorSubTable");
          glCopyColorSubTable         = (PFNGLCOPYCOLORSUBTABLEPROC)         wglGetProcAddress("glCopyColorSubTable"); 
          glConvolutionFilter1D       = (PFNGLCONVOLUTIONFILTER1DPROC)       wglGetProcAddress("glConvolutionFilter1D");
          glConvolutionFilter2D       = (PFNGLCONVOLUTIONFILTER2DPROC)       wglGetProcAddress("glConvolutionFilter2D");
          glConvolutionParameterf     = (PFNGLCONVOLUTIONPARAMETERFPROC)     wglGetProcAddress("glConvolutionParameterf");
          glConvolutionParameterfv    = (PFNGLCONVOLUTIONPARAMETERFVPROC)    wglGetProcAddress("glConvolutionParameterfv");
          glConvolutionParameteri     = (PFNGLCONVOLUTIONPARAMETERIPROC)     wglGetProcAddress("glConvolutionParameteri");
          glConvolutionParameteriv    = (PFNGLCONVOLUTIONPARAMETERIVPROC)    wglGetProcAddress("glConvolutionParameteriv");
          glCopyConvolutionFilter1D   = (PFNGLCOPYCONVOLUTIONFILTER1DPROC)   wglGetProcAddress("glCopyConvolutionFilter1D");
          glCopyConvolutionFilter2D   = (PFNGLCOPYCONVOLUTIONFILTER2DPROC)   wglGetProcAddress("glCopyConvolutionFilter2D");
          glGetConvolutionFilter      = (PFNGLGETCONVOLUTIONFILTERPROC)      wglGetProcAddress("glGetConvolutionFilter");
          glGetConvolutionParameterfv = (PFNGLGETCONVOLUTIONPARAMETERFVPROC) wglGetProcAddress("glGetConvolutionParameterfv"); 
          glGetConvolutionParameteriv = (PFNGLGETCONVOLUTIONPARAMETERIVPROC) wglGetProcAddress("glGetConvolutionParameteriv");
          glGetSeparableFilter        = (PFNGLGETSEPARABLEFILTERPROC)        wglGetProcAddress("glGetSeparableFilter");
          glSeparableFilter2D         = (PFNGLSEPARABLEFILTER2DPROC)         wglGetProcAddress("glSeparableFilter2D");
          glGetHistogram              = (PFNGLGETHISTOGRAMPROC)              wglGetProcAddress("glGetHistogram");
          glGetHistogramParameterfv   = (PFNGLGETHISTOGRAMPARAMETERFVPROC)   wglGetProcAddress("glGetHistogramParameterfv"); 
          glGetHistogramParameteriv   = (PFNGLGETHISTOGRAMPARAMETERIVPROC)   wglGetProcAddress("glGetHistogramParameteriv");
          glGetMinmax                 = (PFNGLGETMINMAXPROC)                 wglGetProcAddress("glGetMinmax");
          glGetMinmaxParameterfv      = (PFNGLGETMINMAXPARAMETERFVPROC)      wglGetProcAddress("glGetMinmaxParameterfv"); 
          glGetMinmaxParameteriv      = (PFNGLGETMINMAXPARAMETERIVPROC)      wglGetProcAddress("glGetMinmaxParameteriv");
          glHistogram                 = (PFNGLHISTOGRAMPROC)                 wglGetProcAddress("glHistogram");
          glMinmax                    = (PFNGLMINMAXPROC)                    wglGetProcAddress("glMinmax");
          glResetHistogram            = (PFNGLRESETHISTOGRAMPROC)            wglGetProcAddress("glResetHistogram"); 
          glResetMinmax               = (PFNGLRESETMINMAXPROC)               wglGetProcAddress("glResetMinmax");
          glTexImage3D                = (PFNGLTEXIMAGE3DPROC)                wglGetProcAddress("glTexImage3D");
          glTexSubImage3D             = (PFNGLTEXSUBIMAGE3DPROC)             wglGetProcAddress("glTexSubImage3D"); 
          glCopyTexSubImage3D         = (PFNGLCOPYTEXSUBIMAGE3DPROC)         wglGetProcAddress("glCopyTexSubImage3D");
        #endif

        return GL_TRUE;
      }

    return GL_FALSE;
  }

//#####################################################################################
//                                   OPENGL 1.1
//#####################################################################################

GLboolean initGL_1_1(void) 
  {
    char *Version = (char *)glGetString(GL_VERSION);
    if(strncmp(Version,"1.1",3) == 0)
      {
        return GL_TRUE;
      }
    return GL_FALSE;
  }

//#####################################################################################
//              EXTENSION INITIALISATION - GETTING THE RUNTIME ADDRESSES
//#####################################################################################

void initExtension( char *ExtName )

  {
    #ifdef GL_ARB_multitexture
    if(strcmp("GL_ARB_multitexture",ExtName) == 0 )
      {
        glActiveTextureARB       = (PFNGLACTIVETEXTUREARBPROC)       wglGetProcAddress("glActiveTextureARB");
        glClientActiveTextureARB = (PFNGLCLIENTACTIVETEXTUREARBPROC) wglGetProcAddress("glClientActiveTextureARB");	
        glMultiTexCoord1dARB     = (PFNGLMULTITEXCOORD1DARBPROC)     wglGetProcAddress("glMultiTexCoord1dARB");
        glMultiTexCoord1dvARB    = (PFNGLMULTITEXCOORD1DVARBPROC)    wglGetProcAddress("glMultiTexCoord1dvARB");
        glMultiTexCoord1fARB     = (PFNGLMULTITEXCOORD1FARBPROC)     wglGetProcAddress("glMultiTexCoord1fARB");
        glMultiTexCoord1fvARB    = (PFNGLMULTITEXCOORD1FVARBPROC)    wglGetProcAddress("glMultiTexCoord1fvARB");
        glMultiTexCoord1iARB     = (PFNGLMULTITEXCOORD1IARBPROC)     wglGetProcAddress("glMultiTexCoord1iARB");
        glMultiTexCoord1ivARB    = (PFNGLMULTITEXCOORD1IVARBPROC)    wglGetProcAddress("glMultiTexCoord1ivARB");
        glMultiTexCoord1sARB     = (PFNGLMULTITEXCOORD1SARBPROC)     wglGetProcAddress("glMultiTexCoord1sARB");
        glMultiTexCoord1svARB    = (PFNGLMULTITEXCOORD1SVARBPROC)    wglGetProcAddress("glMultiTexCoord1svARB");
        glMultiTexCoord2dARB     = (PFNGLMULTITEXCOORD2DARBPROC)     wglGetProcAddress("glMultiTexCoord2dARB");
        glMultiTexCoord2dvARB    = (PFNGLMULTITEXCOORD2DVARBPROC)    wglGetProcAddress("glMultiTexCoord2dvARB");
        glMultiTexCoord2fARB     = (PFNGLMULTITEXCOORD2FARBPROC)     wglGetProcAddress("glMultiTexCoord2fARB");
        glMultiTexCoord2fvARB    = (PFNGLMULTITEXCOORD2FVARBPROC)    wglGetProcAddress("glMultiTexCoord2fvARB");
        glMultiTexCoord2iARB     = (PFNGLMULTITEXCOORD2IARBPROC)     wglGetProcAddress("glMultiTexCoord2iARB");
        glMultiTexCoord2ivARB    = (PFNGLMULTITEXCOORD2IVARBPROC)    wglGetProcAddress("glMultiTexCoord2ivARB");
        glMultiTexCoord2sARB     = (PFNGLMULTITEXCOORD2SARBPROC)     wglGetProcAddress("glMultiTexCoord2sARB");
        glMultiTexCoord2svARB    = (PFNGLMULTITEXCOORD2SVARBPROC)    wglGetProcAddress("glMultiTexCoord2svARB");
        glMultiTexCoord3dARB     = (PFNGLMULTITEXCOORD3DARBPROC)     wglGetProcAddress("glMultiTexCoord3dARB");
        glMultiTexCoord3dvARB    = (PFNGLMULTITEXCOORD3DVARBPROC)    wglGetProcAddress("glMultiTexCoord3dvARB");
        glMultiTexCoord3fARB     = (PFNGLMULTITEXCOORD3FARBPROC)     wglGetProcAddress("glMultiTexCoord3fARB");
        glMultiTexCoord3fvARB    = (PFNGLMULTITEXCOORD3FVARBPROC)    wglGetProcAddress("glMultiTexCoord3fvARB");
        glMultiTexCoord3iARB     = (PFNGLMULTITEXCOORD3IARBPROC)     wglGetProcAddress("glMultiTexCoord3iARB");
        glMultiTexCoord3ivARB    = (PFNGLMULTITEXCOORD3IVARBPROC)    wglGetProcAddress("glMultiTexCoord3ivARB");
        glMultiTexCoord3sARB     = (PFNGLMULTITEXCOORD3SARBPROC)     wglGetProcAddress("glMultiTexCoord3sARB");
        glMultiTexCoord3svARB    = (PFNGLMULTITEXCOORD3SVARBPROC)    wglGetProcAddress("glMultiTexCoord3svARB");
        glMultiTexCoord4dARB     = (PFNGLMULTITEXCOORD4DARBPROC)     wglGetProcAddress("glMultiTexCoord4dARB");
        glMultiTexCoord4dvARB    = (PFNGLMULTITEXCOORD4DVARBPROC)    wglGetProcAddress("glMultiTexCoord4dvARB");
        glMultiTexCoord4fARB     = (PFNGLMULTITEXCOORD4FARBPROC)     wglGetProcAddress("glMultiTexCoord4fARB");
        glMultiTexCoord4fvARB    = (PFNGLMULTITEXCOORD4FVARBPROC)    wglGetProcAddress("glMultiTexCoord4fvARB");
        glMultiTexCoord4iARB     = (PFNGLMULTITEXCOORD4IARBPROC)     wglGetProcAddress("glMultiTexCoord4iARB");
        glMultiTexCoord4ivARB    = (PFNGLMULTITEXCOORD4IVARBPROC)    wglGetProcAddress("glMultiTexCoord4ivARB");
        glMultiTexCoord4sARB     = (PFNGLMULTITEXCOORD4SARBPROC)     wglGetProcAddress("glMultiTexCoord4sARB");
        glMultiTexCoord4svARB    = (PFNGLMULTITEXCOORD4SVARBPROC)    wglGetProcAddress("glMultiTexCoord4svARB");
        return;
      }
    #endif

    #ifdef GL_ARB_transpose_matrix
    if( strcmp("GL_ARB_transpose_matrix",ExtName) == 0 )
      {
        glLoadTransposeMatrixfARB = (PFNGLLOADTRANSPOSEMATRIXFARBPROC) wglGetProcAddress("glLoadTransposeMatrixfARB");
        glLoadTransposeMatrixdARB = (PFNGLLOADTRANSPOSEMATRIXDARBPROC) wglGetProcAddress("glLoadTransposeMatrixdARB");
        glMultTransposeMatrixfARB = (PFNGLMULTTRANSPOSEMATRIXFARBPROC) wglGetProcAddress("glMultTransposeMatrixfARB");
        glMultTransposeMatrixdARB = (PFNGLMULTTRANSPOSEMATRIXDARBPROC) wglGetProcAddress("glMultTransposeMatrixdARB");
      }
    #endif


    #ifdef GL_ARB_multisample
    if( strcmp("GL_ARB_multisample",ExtName) == 0 )
      {
        glSampleCoverageARB = (PFNGLSAMPLECOVERAGEARBPROC)  wglGetProcAddress("glSampleCoverageARB");
        glSamplePassARB     = (PFNGLSAMPLEPASSARBPROC)      wglGetProcAddress("glSamplePassARB");
      }
    #endif


    #ifdef GL_ARB_texture_compression
    if( strcmp("GL_ARB_texture_compression",ExtName) == 0 )
      {
        glCompressedTexImage3DARB     = (PFNGLCOMPRESSEDTEXIMAGE3DARBPROC)      wglGetProcAddress("glCompressedTexImage3DARB");
        glCompressedTexImage2DARB     = (PFNGLCOMPRESSEDTEXIMAGE2DARBPROC)      wglGetProcAddress("glCompressedTexImage2DARB");
        glCompressedTexImage1DARB     = (PFNGLCOMPRESSEDTEXIMAGE1DARBPROC)      wglGetProcAddress("glCompressedTexImage1DARB");
        glCompressedTexSubImage3DARB  = (PFNGLCOMPRESSEDTEXSUBIMAGE3DARBPROC)   wglGetProcAddress("glCompressedTexSubImage3DARB");
        glCompressedTexSubImage2DARB  = (PFNGLCOMPRESSEDTEXSUBIMAGE2DARBPROC)   wglGetProcAddress("glCompressedTexSubImage2DARB");
        glCompressedTexSubImage1DARB  = (PFNGLCOMPRESSEDTEXSUBIMAGE1DARBPROC)   wglGetProcAddress("glCompressedTexSubImage1DARB");
        glGetCompressedTexImageARB    = (PFNGLGETCOMPRESSEDTEXIMAGEARBPROC)     wglGetProcAddress("glGetCompressedTexImageARB");
      }
    #endif

    #ifdef GL_EXT_blend_color
    if( strcmp("GL_EXT_blend_color",ExtName) == 0 )
      {
        glBlendColorEXT  = (PFNGLBLENDCOLOREXTPROC)  wglGetProcAddress("glBlendColorEXT");
      }
    #endif

    #ifdef GL_EXT_polygon_offset
    if( strcmp("GL_EXT_polygon_offset",ExtName) == 0 )
      {
        glPolygonOffsetEXT = (PFNGLPOLYGONOFFSETEXTPROC) wglGetProcAddress("glPolygonOffsetEXT");
      }
    #endif

    #ifdef GL_EXT_texture3D
    if( strcmp("GL_EXT_texture3D",ExtName) == 0 )
      {
        glTexImage3DEXT    = (PFNGLTEXIMAGE3DEXTPROC)     wglGetProcAddress("glTexImage3DEXT");
        glTexSubImage3DEXT = (PFNGLTEXSUBIMAGE3DEXTPROC)  wglGetProcAddress("glTexSubImage3DEXT");
      }
    #endif

    #ifdef GL_EXT_histogram
    if( strcmp("GL_EXT_histogram",ExtName) == 0 )
      {
        glGetHistogramEXT             = (PFNGLGETHISTOGRAMEXTPROC)             wglGetProcAddress("glGetHistogramEXT");
        glGetHistogramParameterfvEXT  = (PFNGLGETHISTOGRAMPARAMETERFVEXTPROC)  wglGetProcAddress("glGetHistogramParameterfvEXT");
        glGetHistogramParameterivEXT  = (PFNGLGETHISTOGRAMPARAMETERIVEXTPROC)  wglGetProcAddress("glGetHistogramParameterivEXT");
        glGetMinmaxEXT                = (PFNGLGETMINMAXEXTPROC)                wglGetProcAddress("glGetMinmaxEXT");
        glGetMinmaxParameterfvEXT     = (PFNGLGETMINMAXPARAMETERFVEXTPROC)     wglGetProcAddress("glGetMinmaxParameterfvEXT");
        glGetMinmaxParameterivEXT     = (PFNGLGETMINMAXPARAMETERIVEXTPROC)     wglGetProcAddress("glGetMinmaxParameterivEXT");
        glHistogramEXT                = (PFNGLHISTOGRAMEXTPROC)                wglGetProcAddress("glHistogramEXT");
        glMinmaxEXT                   = (PFNGLMINMAXEXTPROC)                   wglGetProcAddress("glMinmaxEXT");
        glResetHistogramEXT           = (PFNGLRESETHISTOGRAMEXTPROC)           wglGetProcAddress("glResetHistogramEXT");
        glResetMinmaxEXT              = (PFNGLRESETMINMAXEXTPROC)              wglGetProcAddress("glResetMinmaxEXT");
      }
    #endif


    #ifdef GL_EXT_convolution
    if( strcmp("GL_EXT_convolution",ExtName) == 0 )
      {
        glConvolutionFilter1DEXT       = (PFNGLCONVOLUTIONFILTER1DEXTPROC)       wglGetProcAddress("glConvolutionFilter1DEXT");
        glConvolutionFilter2DEXT       = (PFNGLCONVOLUTIONFILTER2DEXTPROC)       wglGetProcAddress("glConvolutionFilter2DEXT");
        glConvolutionParameterfEXT     = (PFNGLCONVOLUTIONPARAMETERFEXTPROC)     wglGetProcAddress("glConvolutionParameterfEXT");
        glConvolutionParameterfvEXT    = (PFNGLCONVOLUTIONPARAMETERFVEXTPROC)    wglGetProcAddress("glConvolutionParameterfvEXT");
        glConvolutionParameteriEXT     = (PFNGLCONVOLUTIONPARAMETERIEXTPROC)     wglGetProcAddress("glConvolutionParameteriEXT");
        glConvolutionParameterivEXT    = (PFNGLCONVOLUTIONPARAMETERIVEXTPROC)    wglGetProcAddress("glConvolutionParameterivEXT");
        glCopyConvolutionFilter1DEXT   = (PFNGLCOPYCONVOLUTIONFILTER1DEXTPROC)   wglGetProcAddress("glCopyConvolutionFilter1DEXT");
        glCopyConvolutionFilter2DEXT   = (PFNGLCOPYCONVOLUTIONFILTER2DEXTPROC)   wglGetProcAddress("glCopyConvolutionFilter2DEXT");
        glGetConvolutionFilterEXT      = (PFNGLGETCONVOLUTIONFILTEREXTPROC)      wglGetProcAddress("glGetConvolutionFilterEXT");
        glGetConvolutionParameterfvEXT = (PFNGLGETCONVOLUTIONPARAMETERFVEXTPROC) wglGetProcAddress("glGetConvolutionParameterfvEXT");
        glGetConvolutionParameterivEXT = (PFNGLGETCONVOLUTIONPARAMETERIVEXTPROC) wglGetProcAddress("glGetConvolutionParameterivEXT");
        glGetSeparableFilterEXT        = (PFNGLGETSEPARABLEFILTEREXTPROC)        wglGetProcAddress("glGetSeparableFilterEXT");
        glSeparableFilter2DEXT         = (PFNGLSEPARABLEFILTER2DEXTPROC)         wglGetProcAddress("glSeparableFilter2DEXT");
      }
    #endif


    #ifndef GL_EXT_texture_object
    if( strcmp("GL_EXT_texture_object",ExtName) == 0 )
      {
        glAreTexturesResidentEXT = (PFNGLARETEXTURESRESIDENTEXTPROC) wglGetProcAddress("glAreTexturesResidentEXT");
        glBindTextureEXT         = (PFNGLBINDTEXTUREEXTPROC)         wglGetProcAddress("glBindTextureEXT");
        glDeleteTexturesEXT      = (PFNGLDELETETEXTURESEXTPROC)      wglGetProcAddress("glDeleteTexturesEXT");
        glGenTexturesEXT         = (PFNGLGENTEXTURESEXTPROC)         wglGetProcAddress("glGenTexturesEXT");
        glIsTextureEXT           = (PFNGLISTEXTUREEXTPROC)           wglGetProcAddress("glIsTextureEXT");
        glPrioritizeTexturesEXT  = (PFNGLPRIORITIZETEXTURESEXTPROC)  wglGetProcAddress("glPrioritizeTexturesEXT");
      }
    #endif

    #ifdef GL_EXT_vertex_array
    if( strcmp("GL_EXT_vertex_array",ExtName) == 0 )
      {
        glArrayElementEXT    = (PFNGLARRAYELEMENTEXTPROC)    wglGetProcAddress("glArrayElementEXT");
        glColorPointerEXT    = (PFNGLCOLORPOINTEREXTPROC)    wglGetProcAddress("glColorPointerEXT");
        glDrawArraysEXT      = (PFNGLDRAWARRAYSEXTPROC)      wglGetProcAddress("glDrawArraysEXT");
        glEdgeFlagPointerEXT = (PFNGLEDGEFLAGPOINTEREXTPROC) wglGetProcAddress("glEdgeFlagPointerEXT");
        glGetPointervEXT     = (PFNGLGETPOINTERVEXTPROC)     wglGetProcAddress("glGetPointervEXT");
        glIndexPointerEXT    = (PFNGLINDEXPOINTEREXTPROC)    wglGetProcAddress("glIndexPointerEXT");
        glNormalPointerEXT   = (PFNGLNORMALPOINTEREXTPROC)   wglGetProcAddress("glNormalPointerEXT");
        glTexCoordPointerEXT = (PFNGLTEXCOORDPOINTEREXTPROC) wglGetProcAddress("glTexCoordPointerEXT");
        glVertexPointerEXT   = (PFNGLVERTEXPOINTEREXTPROC)   wglGetProcAddress("glVertexPointerEXT");
      }
    #endif

    #ifdef GL_EXT_blend_minmax
    if( strcmp("GL_EXT_blend_minmax",ExtName) == 0 )
      {
        glBlendEquationEXT             = (PFNGLBLENDEQUATIONEXTPROC)              wglGetProcAddress("glBlendEquationEXT");
      }
    #endif

    #ifdef GL_EXT_point_parameters
    if( strcmp("GL_EXT_point_parameters",ExtName) == 0 )
      {
        glPointParameterfEXT           = (PFNGLPOINTPARAMETERFEXTPROC)            wglGetProcAddress("glPointParameterfEXT");
        glPointParameterfvEXT          = (PFNGLPOINTPARAMETERFVEXTPROC)           wglGetProcAddress("glPointParameterfvEXT");
        glPointParameterfSGIS          = (PFNGLPOINTPARAMETERFSGISPROC)           wglGetProcAddress("glPointParameterfSGIS");
        glPointParameterfvSGIS         = (PFNGLPOINTPARAMETERFVSGISPROC)          wglGetProcAddress("glPointParameterfvSGIS");
      }
    #endif

    #ifdef GL_EXT_light_texture
    if( strcmp("GL_EXT_light_texture",ExtName) == 0 )
      {
        glApplyTextureEXT              = (PFNGLAPPLYTEXTUREEXTPROC)               wglGetProcAddress("glApplyTextureEXT");
        glTextureLightEXT              = (PFNGLTEXTURELIGHTEXTPROC)               wglGetProcAddress("glTextureLightEXT");
        glTextureMaterialEXT           = (PFNGLTEXTUREMATERIALEXTPROC)            wglGetProcAddress("glTextureMaterialEXT");
      }
    #endif

    #ifdef GL_EXT_index_material
    if( strcmp("GL_EXT_index_material",ExtName) == 0 )
      {
        glIndexMaterialEXT             = (PFNGLINDEXMATERIALEXTPROC)              wglGetProcAddress("glIndexMaterialEXT");
      }
    #endif

    #ifdef GL_EXT_index_func
    if( strcmp("GL_EXT_index_func",ExtName) == 0 )
      {
        glIndexFuncEXT                 = (PFNGLINDEXFUNCEXTPROC)                  wglGetProcAddress("glIndexFuncEXT");
      }
    #endif

    #ifdef GL_EXT_compiled_vertex_array
    if( strcmp("GL_EXT_compiled_vertex_array",ExtName) == 0 )
      {
        glLockArraysEXT                = (PFNGLLOCKARRAYSEXTPROC)                 wglGetProcAddress("glLockArraysEXT");
        glUnlockArraysEXT              = (PFNGLUNLOCKARRAYSEXTPROC)               wglGetProcAddress("glUnlockArraysEXT");
      }
    #endif

    #ifdef GL_EXT_cull_vertex
    if( strcmp("GL_EXT_cull_vertex",ExtName) == 0 )
      {
        glCullParameterdvEXT           = (PFNGLCULLPARAMETERDVEXTPROC)            wglGetProcAddress("glCullParameterdvEXT");
        glCullParameterfvEXT           = (PFNGLCULLPARAMETERFVEXTPROC)            wglGetProcAddress("glCullParameterfvEXT");
      }
    #endif

    #ifdef GL_EXT_draw_range_elements
    if( strcmp("GL_EXT_draw_range_elements",ExtName) == 0 )
      {
        glDrawRangeElementsEXT         = (PFNGLDRAWRANGEELEMENTSEXTPROC)          wglGetProcAddress("glDrawRangeElementsEXT");
      }
    #endif

    #ifdef GL_EXT_pixel_transform
    if( strcmp("GL_EXT_pixel_transform",ExtName) == 0 )
      {
        glPixelTransformParameteriEXT  = (PFNGLPIXELTRANSFORMPARAMETERIEXTPROC)   wglGetProcAddress("glPixelTransformParameteriEXT");
        glPixelTransformParameterfEXT  = (PFNGLPIXELTRANSFORMPARAMETERFEXTPROC)   wglGetProcAddress("glPixelTransformParameterfEXT");
        glPixelTransformParameterivEXT = (PFNGLPIXELTRANSFORMPARAMETERIVEXTPROC)  wglGetProcAddress("glPixelTransformParameterivEXT");
        glPixelTransformParameterfvEXT = (PFNGLPIXELTRANSFORMPARAMETERFVEXTPROC)  wglGetProcAddress("glPixelTransformParameterfvEXT");
      }
    #endif

    #ifdef GL_EXT_secondary_color
    if( strcmp("GL_EXT_secondary_color",ExtName) == 0 )
      {
        glSecondaryColor3bEXT          = (PFNGLSECONDARYCOLOR3BEXTPROC)           wglGetProcAddress("glSecondaryColor3bEXT");
        glSecondaryColor3bvEXT         = (PFNGLSECONDARYCOLOR3BVEXTPROC)          wglGetProcAddress("glSecondaryColor3bvEXT");
        glSecondaryColor3dEXT          = (PFNGLSECONDARYCOLOR3DEXTPROC)           wglGetProcAddress("glSecondaryColor3dEXT");
        glSecondaryColor3dvEXT         = (PFNGLSECONDARYCOLOR3DVEXTPROC)          wglGetProcAddress("glSecondaryColor3dvEXT");
        glSecondaryColor3fEXT          = (PFNGLSECONDARYCOLOR3FEXTPROC)           wglGetProcAddress("glSecondaryColor3fEXT");
        glSecondaryColor3fvEXT         = (PFNGLSECONDARYCOLOR3FVEXTPROC)          wglGetProcAddress("glSecondaryColor3fvEXT");
        glSecondaryColor3iEXT          = (PFNGLSECONDARYCOLOR3IEXTPROC)           wglGetProcAddress("glSecondaryColor3iEXT");
        glSecondaryColor3ivEXT         = (PFNGLSECONDARYCOLOR3IVEXTPROC)          wglGetProcAddress("glSecondaryColor3ivEXT");
        glSecondaryColor3sEXT          = (PFNGLSECONDARYCOLOR3SEXTPROC)           wglGetProcAddress("glSecondaryColor3sEXT");
        glSecondaryColor3svEXT         = (PFNGLSECONDARYCOLOR3SVEXTPROC)          wglGetProcAddress("glSecondaryColor3svEXT");
        glSecondaryColor3ubEXT         = (PFNGLSECONDARYCOLOR3UBEXTPROC)          wglGetProcAddress("glSecondaryColor3ubEXT");
        glSecondaryColor3ubvEXT        = (PFNGLSECONDARYCOLOR3UBVEXTPROC)         wglGetProcAddress("glSecondaryColor3ubvEXT");
        glSecondaryColor3uiEXT         = (PFNGLSECONDARYCOLOR3UIEXTPROC)          wglGetProcAddress("glSecondaryColor3uiEXT");
        glSecondaryColor3uivEXT        = (PFNGLSECONDARYCOLOR3UIVEXTPROC)         wglGetProcAddress("glSecondaryColor3uivEXT");
        glSecondaryColor3usEXT         = (PFNGLSECONDARYCOLOR3USEXTPROC)          wglGetProcAddress("glSecondaryColor3usEXT");
        glSecondaryColor3usvEXT        = (PFNGLSECONDARYCOLOR3USVEXTPROC)         wglGetProcAddress("glSecondaryColor3usvEXT");
        glSecondaryColorPointerEXT     = (PFNGLSECONDARYCOLORPOINTEREXTPROC)      wglGetProcAddress("glSecondaryColorPointerEXT");
      }
    #endif

    #ifdef GL_EXT_texture_perturb_normal
    if( strcmp("GL_EXT_texture_perturb_normal",ExtName) == 0 )
      {
        glTextureNormalEXT  = (PFNGLTEXTURENORMALEXTPROC)  wglGetProcAddress("glTextureNormalEXT");
      }
    #endif

    #ifdef GL_EXT_fog_coord
    if( strcmp("GL_EXT_fog_coord",ExtName) == 0 )
      {
        glFogCoordfEXT       = (PFNGLFOGCOORDFEXTPROC)       wglGetProcAddress("glFogCoordfEXT");
        glFogCoordfvEXT      = (PFNGLFOGCOORDFVEXTPROC)      wglGetProcAddress("glFogCoordfvEXT");
        glFogCoorddEXT       = (PFNGLFOGCOORDDEXTPROC)       wglGetProcAddress("glFogCoorddEXT");
        glFogCoorddvEXT      = (PFNGLFOGCOORDDVEXTPROC)      wglGetProcAddress("glFogCoorddvEXT");
        glFogCoordPointerEXT = (PFNGLFOGCOORDPOINTEREXTPROC) wglGetProcAddress("glFogCoordPointerEXT");
      }
    #endif

    #ifdef GL_EXT_blend_func_separate
    if( strcmp("GL_EXT_blend_func_separate",ExtName) == 0 )
      {
        glBlendFuncSeparateEXT = (PFNGLBLENDFUNCSEPARATEEXTPROC) wglGetProcAddress("glBlendFuncSeparateEXT");
      }
    #endif

    #ifdef GL_EXT_vertex_weighting
    if( strcmp("GL_EXT_vertex_weighting",ExtName) == 0 )
      {
        glVertexWeightfEXT       = (PFNGLVERTEXWEIGHTFEXTPROC)       wglGetProcAddress("glVertexWeightfEXT");
        glVertexWeightfvEXT      = (PFNGLVERTEXWEIGHTFVEXTPROC)      wglGetProcAddress("glVertexWeightfvEXT");
        glVertexWeightPointerEXT = (PFNGLVERTEXWEIGHTPOINTEREXTPROC) wglGetProcAddress("glVertexWeightPointerEXT");
      }
    #endif


    #ifdef GL_EXT_convolution
    if( strcmp("GL_EXT_convolution",ExtName) == 0 )
      {
        glConvolutionFilter1DEXT       = (PFNGLCONVOLUTIONFILTER1DEXTPROC)       wglGetProcAddress("glConvolutionFilter1DEXT"); 
        glConvolutionFilter2DEXT       = (PFNGLCONVOLUTIONFILTER2DEXTPROC)       wglGetProcAddress("glConvolutionFilter2DEXT"); 
        glConvolutionParameterfEXT     = (PFNGLCONVOLUTIONPARAMETERFEXTPROC)     wglGetProcAddress("glConvolutionParameterfEXT"); 
        glConvolutionParameterfvEXT    = (PFNGLCONVOLUTIONPARAMETERFVEXTPROC)    wglGetProcAddress("glConvolutionParameterfvEXT"); 
        glConvolutionParameteriEXT     = (PFNGLCONVOLUTIONPARAMETERIEXTPROC)     wglGetProcAddress("glConvolutionParameteriEXT"); 
        glConvolutionParameterivEXT    = (PFNGLCONVOLUTIONPARAMETERIVEXTPROC)    wglGetProcAddress("glConvolutionParameterivEXT"); 
        glCopyConvolutionFilter1DEXT   = (PFNGLCOPYCONVOLUTIONFILTER1DEXTPROC)   wglGetProcAddress("glCopyConvolutionFilter1DEXT"); 
        glCopyConvolutionFilter2DEXT   = (PFNGLCOPYCONVOLUTIONFILTER2DEXTPROC)   wglGetProcAddress("glCopyConvolutionFilter2DEXT"); 
        glGetConvolutionFilterEXT      = (PFNGLGETCONVOLUTIONFILTEREXTPROC)      wglGetProcAddress("glGetConvolutionFilterEXT"); 
        glGetConvolutionParameterfvEXT = (PFNGLGETCONVOLUTIONPARAMETERFVEXTPROC) wglGetProcAddress("glGetConvolutionParameterfvEXT"); 
        glGetConvolutionParameterivEXT = (PFNGLGETCONVOLUTIONPARAMETERIVEXTPROC) wglGetProcAddress("glGetConvolutionParameterivEXT"); 
        glGetSeparableFilterEXT        = (PFNGLGETSEPARABLEFILTEREXTPROC)        wglGetProcAddress("glGetSeparableFilterEXT"); 
        glSeparableFilter2DEXT         = (PFNGLSEPARABLEFILTER2DEXTPROC)         wglGetProcAddress("glSeparableFilter2DEXT"); 
      }
    #endif

    #ifdef GL_NV_register_combiners
    if( strcmp("GL_NV_register_combiners",ExtName) == 0 )
      {
        glCombinerParameterfvNV              = (PFNGLCOMBINERPARAMETERFVNVPROC)               wglGetProcAddress("glCombinerParameterfvNV"); 
        glCombinerParameterfNV               = (PFNGLCOMBINERPARAMETERFNVPROC)                wglGetProcAddress("glCombinerParameterfNV");
        glCombinerParameterivNV              = (PFNGLCOMBINERPARAMETERIVNVPROC)               wglGetProcAddress("glCombinerParameterivNV");
        glCombinerParameteriNV               = (PFNGLCOMBINERPARAMETERINVPROC)                wglGetProcAddress("glCombinerParameteriNV");
        glCombinerInputNV                    = (PFNGLCOMBINERINPUTNVPROC)                     wglGetProcAddress("glCombinerInputNV");
        glCombinerOutputNV                   = (PFNGLCOMBINEROUTPUTNVPROC)                    wglGetProcAddress("glCombinerOutputNV");
        glFinalCombinerInputNV               = (PFNGLFINALCOMBINERINPUTNVPROC)                wglGetProcAddress("glFinalCombinerInputNV");
        glGetCombinerInputParameterfvNV      = (PFNGLGETCOMBINERINPUTPARAMETERFVNVPROC)       wglGetProcAddress("glGetCombinerInputParameterfvNV");
        glGetCombinerInputParameterivNV      = (PFNGLGETCOMBINERINPUTPARAMETERIVNVPROC)       wglGetProcAddress("glGetCombinerInputParameterivNV");
        glGetCombinerOutputParameterfvNV     = (PFNGLGETCOMBINEROUTPUTPARAMETERFVNVPROC)      wglGetProcAddress("glGetCombinerOutputParameterfvNV");
        glGetCombinerOutputParameterivNV     = (PFNGLGETCOMBINEROUTPUTPARAMETERIVNVPROC)      wglGetProcAddress("glGetCombinerOutputParameterivNV");
        glGetFinalCombinerInputParameterfvNV = (PFNGLGETFINALCOMBINERINPUTPARAMETERFVNVPROC)  wglGetProcAddress("glGetFinalCombinerInputParameterfvNV");
        glGetFinalCombinerInputParameterivNV = (PFNGLGETFINALCOMBINERINPUTPARAMETERIVNVPROC)  wglGetProcAddress("glGetFinalCombinerInputParameterivNV");
      }
    #endif

    #ifdef GL_NV_vertex_array_range
    if( strcmp("GL_NV_vertex_array_range",ExtName) == 0 )
      {
        glFlushVertexArrayRangeNV = (PFNGLFLUSHVERTEXARRAYRANGENVPROC) wglGetProcAddress("glFlushVertexArrayRangeNV");
        glVertexArrayRangeNV      = (PFNGLVERTEXARRAYRANGENVPROC)      wglGetProcAddress("glVertexArrayRangeNV");
      }
    #endif

    #ifdef GL_EXT_multisample
    if( strcmp("GL_EXT_multisample",ExtName) == 0 )
      {
        glSampleMaskEXT    = (PFNGLSAMPLEMASKEXTPROC)    wglGetProcAddress("glSampleMaskEXT");
        glSamplePatternEXT = (PFNGLSAMPLEPATTERNEXTPROC) wglGetProcAddress("glSamplePatternEXT");
      }
    #endif


  }


//#####################################################################################
//              CHECKS EXTENSION STRINGS TO LOOK FOR REQUIRED EXTENSION
//#####################################################################################

GLboolean isInString(char *string, const char *search) 
  {
    int pos , i;
    int maxpos = strlen(search) - 1 ;
    int len = strlen( string );

    for (i=0; i<len; i++)
      {
        if( (i==0) || ((i>1) && string[i-1]=='\n') )
          {
            pos=0;
            while (string[i]!='\n')
              {
                if( string[i]==search[pos] ) pos++;
                if( (pos>maxpos) && string[i+1]=='\n' ) return GL_TRUE; 
                i++;
              }			
          }
      }

    return GL_FALSE;   // Not Found!
  }

//#####################################################################################
//                           CHECK GL EXTENSION FUNCTION
//#####################################################################################

GLboolean CheckGLExt( char *extString )
  {
    int        length     ,
               i          ;
    char      *extensions ;

    if(strcmp(extString,"GL_VERSION_1_2") == 0) return initGL_1_2();
    else 
    if(strcmp(extString,"GL_VERSION_1_1") == 0) return initGL_1_1();
    else 
      {

        extensions = (char *) glGetString(GL_EXTENSIONS);
        length = strlen(extensions);
        for ( i=0; i<length; i++ )
          {
            if( extensions[i] == ' ' ) extensions[i]='\n';
          }
        if(  isInString( extensions , extString )  )
          {
            initExtension( extString );
            return GL_TRUE;
          }

      }
    return GL_FALSE;

  }